@@ -1,3 +1,22 @@
+/*
+  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
+  2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+
+  This file is part of GNU Inetutils.
+
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
+
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
+
 /*
  * Copyright (c) 1985, 1989, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
@@ -10,7 +29,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -27,26 +46,6 @@
  * SUCH DAMAGE.
  */
 
-/* Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
-   Free Software Foundation, Inc.
-
-   This file is part of GNU Inetutils.
-
-   GNU Inetutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
-
-   GNU Inetutils is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write
-   to the Free Software Foundation, Inc., 51 Franklin Street,
-   Fifth Floor, Boston, MA 02110-1301 USA. */
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -327,8 +326,7 @@ login (host)
     {
       if (!strcmp ("init", macros[n].mac_name))
 	{
-	  if (line)
-	    free (line);
+	  free (line);
 	  line = calloc (200, sizeof (*line));
 	  if (!line)
 	    quit (0, 0);
@@ -965,8 +963,7 @@ recvrequest (cmd, local, remote, lmode, printnames)
   blksize = BUFSIZ;
   if (blksize > bufsize)
     {
-      if (buf)
-	free (buf);
+      free (buf);
       buf = malloc ((unsigned) blksize);
       if (buf == NULL)
 	{
@@ -1421,8 +1418,7 @@ pswitch (flag)
   ip->connect = connected;
   connected = op->connect;
 
-  if (ip->name)
-    free (ip->name);
+  free (ip->name);
   ip->name = hostname;
   hostname = op->name;
   op->name = 0;
@@ -1458,14 +1454,12 @@ pswitch (flag)
   ip->mapflg = mapflag;
   mapflag = op->mapflg;
 
-  if (ip->mi)
-    free (ip->mi);
+  free (ip->mi);
   ip->mi = mapin;
   mapin = op->mi;
   op->mi = 0;
 
-  if (ip->mo)
-    free (ip->mo);
+  free (ip->mo);
   ip->mo = mapout;
   mapout = op->mo;
   op->mo = 0;
@@ -1656,8 +1650,7 @@ gunique (local)
   int count = 0;
   char ext = '1';
 
-  if (new)
-    free (new);
+  free (new);
   new = malloc (strlen (local) + 1 + 3 + 1);	/* '.' + 100 + '\0' */
   if (!new)
     {
