@@ -1,3 +1,22 @@
+/*
+  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
+  2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+
+  This file is part of GNU Inetutils.
+
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
+
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
+
 /*
  * Copyright (c) 1985, 1989, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
@@ -10,7 +29,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -27,26 +46,6 @@
  * SUCH DAMAGE.
  */
 
-/* Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
-   Free Software Foundation, Inc.
-
-   This file is part of GNU Inetutils.
-
-   GNU Inetutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
-
-   GNU Inetutils is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write
-   to the Free Software Foundation, Inc., 51 Franklin Street,
-   Fifth Floor, Boston, MA 02110-1301 USA. */
-
 /*
  * FTP User Program -- Command Interface.
  */
@@ -88,23 +87,21 @@
 #define DEFAULT_PROMPT "ftp> "
 static char *prompt = 0;
 
-ARGP_PROGRAM_DATA_SIMPLE ("ftp", "2007");
-
 const char args_doc[] = "[HOST [PORT]]";
 const char doc[] = "Remote file transfer.";
 
 static struct argp_option argp_options[] = {
 #define GRP 0
-  {"debug", 'd', NULL, 0, "Set the SO_DEBUG option", GRP+1},
-  {"no-glob", 'g', NULL, 0, "Turn off file name globbing", GRP+1},
-  {"no-prompt", 'i', NULL, 0, "Don't prompt during multiple file transfers",
+  {"debug", 'd', NULL, 0, "set the SO_DEBUG option", GRP+1},
+  {"no-glob", 'g', NULL, 0, "turn off file name globbing", GRP+1},
+  {"no-prompt", 'i', NULL, 0, "do not prompt during multiple file transfers",
    GRP+1},
-  {"no-login", 'n', NULL, 0, "Don't automatically login to the remote system",
+  {"no-login", 'n', NULL, 0, "do not automatically login to the remote system",
    GRP+1},
-  {"trace", 't', NULL, 0, "Enable packet tracing", GRP+1},
-  {"prompt", 'p', "PROMPT", OPTION_ARG_OPTIONAL, "Print a command line PROMPT "
+  {"trace", 't', NULL, 0, "enable packet tracing", GRP+1},
+  {"prompt", 'p', "PROMPT", OPTION_ARG_OPTIONAL, "print a command line PROMPT "
    "(optionally), even if not on a tty", GRP+1},
-  {"verbose", 'v', NULL, 0, "Verbose output", GRP+1},
+  {"verbose", 'v', NULL, 0, "verbose output", GRP+1},
 #undef GRP
   {NULL}
 };
@@ -171,6 +168,7 @@ main (int argc, char *argv[])
   autologin = 1;
 
   /* Parse command line */
+  iu_argp_init ("ftp", default_program_authors);
   argp_parse (&argp, argc, argv, 0, &index, NULL);
 
   argc -= index;
