@@ -1,24 +1,23 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+
+/* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,7 +28,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -47,7 +46,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.3"
+#define YYBISON_VERSION "2.4.1"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -55,11 +54,119 @@
 /* Pure parsers.  */
 #define YYPURE 0
 
+/* Push parsers.  */
+#define YYPUSH 0
+
+/* Pull parsers.  */
+#define YYPULL 1
+
 /* Using locations.  */
 #define YYLSP_NEEDED 0
 
 
 
+/* Copy the first part of user declarations.  */
+
+/* Line 189 of yacc.c  */
+#line 35 "ftpcmd.y"
+
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <sys/types.h>
+#include <sys/param.h>
+#include <sys/socket.h>
+#include <sys/stat.h>
+
+#include <netinet/in.h>
+#include <arpa/ftp.h>
+
+#include <ctype.h>
+#include <errno.h>
+#include <pwd.h>
+#include <setjmp.h>
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <syslog.h>
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+#include <unistd.h>
+#include <limits.h>
+#ifdef HAVE_SYS_UTSNAME_H
+#include <sys/utsname.h>
+#endif
+/* Include glob.h last, because it may define "const" which breaks
+   system headers on some platforms. */
+#include <glob.h>
+
+#include "extern.h"
+
+#if ! defined (NBBY) && defined (CHAR_BIT)
+#define NBBY CHAR_BIT
+#endif
+
+off_t restart_point;
+
+static char cbuf[512];           /* Command Buffer.  */
+static char *fromname;
+static int cmd_type;
+static int cmd_form;
+static int cmd_bytesz;
+
+struct tab
+{
+  const char	*name;
+  short	token;
+  short	state;
+  short	implemented;	/* 1 if command is implemented */
+  const char	*help;
+};
+
+extern struct tab cmdtab[];
+extern struct tab sitetab[];
+static char *copy         (char *);
+static void help          (struct tab *, char *);
+static struct tab *lookup (struct tab *, char *);
+static void sizecmd       (char *);
+static int yylex          (void);
+static void yyerror       (const char *s);
+
+
+
+/* Line 189 of yacc.c  */
+#line 150 "ftpcmd.c"
+
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+
+/* Enabling verbose error messages.  */
+#ifdef YYERROR_VERBOSE
+# undef YYERROR_VERBOSE
+# define YYERROR_VERBOSE 1
+#else
+# define YYERROR_VERBOSE 0
+#endif
+
+/* Enabling the token table.  */
+#ifndef YYTOKEN_TABLE
+# define YYTOKEN_TABLE 0
+#endif
+
+
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
@@ -199,124 +306,32 @@
 
 
 
-/* Copy the first part of user declarations.  */
-#line 37 "ftpcmd.y"
-
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include <sys/types.h>
-#include <sys/param.h>
-#include <sys/socket.h>
-#include <sys/stat.h>
-
-#include <netinet/in.h>
-#include <arpa/ftp.h>
-
-#include <ctype.h>
-#include <errno.h>
-#include <pwd.h>
-#include <setjmp.h>
-#include <signal.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <syslog.h>
-#ifdef TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# ifdef HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-#include <unistd.h>
-#include <limits.h>
-#ifdef HAVE_SYS_UTSNAME_H
-#include <sys/utsname.h>
-#endif
-/* Include glob.h last, because it may define "const" which breaks
-   system headers on some platforms. */
-#include <glob.h>
-
-#include "extern.h"
-
-#if ! defined (NBBY) && defined (CHAR_BIT)
-#define NBBY CHAR_BIT
-#endif
-
-off_t restart_point;
-
-static char cbuf[512];           /* Command Buffer.  */
-static char *fromname;
-static int cmd_type;
-static int cmd_form;
-static int cmd_bytesz;
-
-struct tab
-{
-  const char	*name;
-  short	token;
-  short	state;
-  short	implemented;	/* 1 if command is implemented */
-  const char	*help;
-};
-
-extern struct tab cmdtab[];
-extern struct tab sitetab[];
-static char *copy         (char *);
-static void help          (struct tab *, char *);
-static struct tab *lookup (struct tab *, char *);
-static void sizecmd       (char *);
-static int yylex          (void);
-static void yyerror       (const char *s);
-
-
-
-/* Enabling traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
-
-/* Enabling verbose error messages.  */
-#ifdef YYERROR_VERBOSE
-# undef YYERROR_VERBOSE
-# define YYERROR_VERBOSE 1
-#else
-# define YYERROR_VERBOSE 0
-#endif
-
-/* Enabling the token table.  */
-#ifndef YYTOKEN_TABLE
-# define YYTOKEN_TABLE 0
-#endif
-
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 113 "ftpcmd.y"
 {
+
+/* Line 214 of yacc.c  */
+#line 111 "ftpcmd.y"
+
 	int	i;
 	char   *s;
-}
-/* Line 187 of yacc.c.  */
-#line 307 "ftpcmd.c"
-	YYSTYPE;
+
+
+
+/* Line 214 of yacc.c  */
+#line 323 "ftpcmd.c"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
-
 /* Copy the second part of user declarations.  */
 
 
-/* Line 216 of yacc.c.  */
-#line 320 "ftpcmd.c"
+/* Line 264 of yacc.c  */
+#line 335 "ftpcmd.c"
 
 #ifdef short
 # undef short
@@ -391,14 +406,14 @@ typedef short int yytype_int16;
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static int
-YYID (int i)
+YYID (int yyi)
 #else
 static int
-YYID (i)
-    int i;
+YYID (yyi)
+    int yyi;
 #endif
 {
-  return i;
+  return yyi;
 }
 #endif
 
@@ -479,9 +494,9 @@ void free (void *); /* INFRINGES ON USER NAME SPACE */
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  yytype_int16 yyss;
-  YYSTYPE yyvs;
-  };
+  yytype_int16 yyss_alloc;
+  YYSTYPE yyvs_alloc;
+};
 
 /* The size of the maximum gap between one aligned stack and the next.  */
 # define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
@@ -515,12 +530,12 @@ union yyalloc
    elements in the stack, and YYPTR gives the new location of the
    stack.  Advance YYPTR to a properly aligned location for the next
    stack.  */
-# define YYSTACK_RELOCATE(Stack)					\
+# define YYSTACK_RELOCATE(Stack_alloc, Stack)				\
     do									\
       {									\
 	YYSIZE_T yynewbytes;						\
-	YYCOPY (&yyptr->Stack, Stack, yysize);				\
-	Stack = &yyptr->Stack;						\
+	YYCOPY (&yyptr->Stack_alloc, Stack, yysize);			\
+	Stack = &yyptr->Stack_alloc;					\
 	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
 	yyptr += yynewbytes / sizeof (*yyptr);				\
       }									\
@@ -642,14 +657,14 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   147,   147,   149,   156,   160,   165,   171,   193,   198,
-     233,   245,   257,   261,   265,   272,   279,   286,   291,   298,
-     303,   310,   317,   321,   328,   341,   345,   350,   357,   361,
-     378,   382,   389,   396,   401,   406,   410,   416,   426,   441,
-     455,   461,   477,   484,   530,   547,   569,   574,   580,   591,
-     606,   611,   614,   618,   622,   636,   640,   644,   651,   656,
-     661,   666,   671,   675,   680,   686,   694,   698,   702,   709,
-     713,   717,   724,   761,   765,   793
+       0,   145,   145,   147,   153,   157,   162,   168,   190,   195,
+     230,   242,   254,   258,   262,   268,   274,   280,   285,   291,
+     296,   302,   308,   312,   318,   331,   335,   340,   346,   350,
+     366,   370,   376,   382,   387,   392,   396,   401,   411,   426,
+     439,   445,   461,   467,   512,   528,   549,   554,   560,   570,
+     584,   589,   592,   596,   600,   614,   618,   622,   629,   634,
+     639,   644,   649,   653,   658,   664,   672,   676,   680,   687,
+     691,   695,   702,   739,   743,   771
 };
 #endif
 
@@ -1048,17 +1063,20 @@ yy_symbol_print (yyoutput, yytype, yyvaluep)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_stack_print (yytype_int16 *bottom, yytype_int16 *top)
+yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
 #else
 static void
-yy_stack_print (bottom, top)
-    yytype_int16 *bottom;
-    yytype_int16 *top;
+yy_stack_print (yybottom, yytop)
+    yytype_int16 *yybottom;
+    yytype_int16 *yytop;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
-  for (; bottom <= top; ++bottom)
-    YYFPRINTF (stderr, " %d", *bottom);
+  for (; yybottom <= yytop; yybottom++)
+    {
+      int yybot = *yybottom;
+      YYFPRINTF (stderr, " %d", yybot);
+    }
   YYFPRINTF (stderr, "\n");
 }
 
@@ -1092,11 +1110,11 @@ yy_reduce_print (yyvsp, yyrule)
   /* The symbols being reduced.  */
   for (yyi = 0; yyi < yynrhs; yyi++)
     {
-      fprintf (stderr, "   $%d = ", yyi + 1);
+      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
 		       &(yyvsp[(yyi + 1) - (yynrhs)])
 		       		       );
-      fprintf (stderr, "\n");
+      YYFPRINTF (stderr, "\n");
     }
 }
 
@@ -1376,10 +1394,8 @@ yydestruct (yymsg, yytype, yyvaluep)
 	break;
     }
 }
-
 
 /* Prevent warnings from -Wmissing-prototypes.  */
-
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
 int yyparse (void *YYPARSE_PARAM);
@@ -1395,11 +1411,10 @@ int yyparse ();
 #endif /* ! YYPARSE_PARAM */
 
 
-
-/* The look-ahead symbol.  */
+/* The lookahead symbol.  */
 int yychar;
 
-/* The semantic value of the look-ahead symbol.  */
+/* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
 /* Number of syntax errors so far.  */
@@ -1407,9 +1422,9 @@ int yynerrs;
 
 
 
-/*----------.
-| yyparse.  |
-`----------*/
+/*-------------------------.
+| yyparse or yypush_parse.  |
+`-------------------------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1433,14 +1448,39 @@ yyparse ()
 #endif
 #endif
 {
-  
-  int yystate;
+
+
+    int yystate;
+    /* Number of tokens to shift before error messages enabled.  */
+    int yyerrstatus;
+
+    /* The stacks and their tools:
+       `yyss': related to states.
+       `yyvs': related to semantic values.
+
+       Refer to the stacks thru separate pointers, to allow yyoverflow
+       to reallocate them elsewhere.  */
+
+    /* The state stack.  */
+    yytype_int16 yyssa[YYINITDEPTH];
+    yytype_int16 *yyss;
+    yytype_int16 *yyssp;
+
+    /* The semantic value stack.  */
+    YYSTYPE yyvsa[YYINITDEPTH];
+    YYSTYPE *yyvs;
+    YYSTYPE *yyvsp;
+
+    YYSIZE_T yystacksize;
+
   int yyn;
   int yyresult;
-  /* Number of tokens to shift before error messages enabled.  */
-  int yyerrstatus;
-  /* Look-ahead token as an internal (translated) token number.  */
-  int yytoken = 0;
+  /* Lookahead token as an internal (translated) token number.  */
+  int yytoken;
+  /* The variables used to return semantic value and location from the
+     action routines.  */
+  YYSTYPE yyval;
+
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
   char yymsgbuf[128];
@@ -1448,51 +1488,28 @@ yyparse ()
   YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
 #endif
 
-  /* Three stacks and their tools:
-     `yyss': related to states,
-     `yyvs': related to semantic values,
-     `yyls': related to locations.
-
-     Refer to the stacks thru separate pointers, to allow yyoverflow
-     to reallocate them elsewhere.  */
-
-  /* The state stack.  */
-  yytype_int16 yyssa[YYINITDEPTH];
-  yytype_int16 *yyss = yyssa;
-  yytype_int16 *yyssp;
-
-  /* The semantic value stack.  */
-  YYSTYPE yyvsa[YYINITDEPTH];
-  YYSTYPE *yyvs = yyvsa;
-  YYSTYPE *yyvsp;
-
-
-
 #define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))
 
-  YYSIZE_T yystacksize = YYINITDEPTH;
-
-  /* The variables used to return semantic value and location from the
-     action routines.  */
-  YYSTYPE yyval;
-
-
   /* The number of symbols on the RHS of the reduced rule.
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
+  yytoken = 0;
+  yyss = yyssa;
+  yyvs = yyvsa;
+  yystacksize = YYINITDEPTH;
+
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yystate = 0;
   yyerrstatus = 0;
   yynerrs = 0;
-  yychar = YYEMPTY;		/* Cause a token to be read.  */
+  yychar = YYEMPTY; /* Cause a token to be read.  */
 
   /* Initialize stack pointers.
      Waste one element of value and location stack
      so that they stay on the same level as the state stack.
      The wasted elements are never initialized.  */
-
   yyssp = yyss;
   yyvsp = yyvs;
 
@@ -1522,7 +1539,6 @@ yyparse ()
 	YYSTYPE *yyvs1 = yyvs;
 	yytype_int16 *yyss1 = yyss;
 
-
 	/* Each stack pointer address is followed by the size of the
 	   data in use in that stack, in bytes.  This used to be a
 	   conditional around just the two extra args, but that might
@@ -1530,7 +1546,6 @@ yyparse ()
 	yyoverflow (YY_("memory exhausted"),
 		    &yyss1, yysize * sizeof (*yyssp),
 		    &yyvs1, yysize * sizeof (*yyvsp),
-
 		    &yystacksize);
 
 	yyss = yyss1;
@@ -1553,9 +1568,8 @@ yyparse ()
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
 	  goto yyexhaustedlab;
-	YYSTACK_RELOCATE (yyss);
-	YYSTACK_RELOCATE (yyvs);
-
+	YYSTACK_RELOCATE (yyss_alloc, yyss);
+	YYSTACK_RELOCATE (yyvs_alloc, yyvs);
 #  undef YYSTACK_RELOCATE
 	if (yyss1 != yyssa)
 	  YYSTACK_FREE (yyss1);
@@ -1566,7 +1580,6 @@ yyparse ()
       yyssp = yyss + yysize - 1;
       yyvsp = yyvs + yysize - 1;
 
-
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
 		  (unsigned long int) yystacksize));
 
@@ -1576,6 +1589,9 @@ yyparse ()
 
   YYDPRINTF ((stderr, "Entering state %d\n", yystate));
 
+  if (yystate == YYFINAL)
+    YYACCEPT;
+
   goto yybackup;
 
 /*-----------.
@@ -1584,16 +1600,16 @@ yyparse ()
 yybackup:
 
   /* Do appropriate processing given the current state.  Read a
-     look-ahead token if we need one and don't already have one.  */
+     lookahead token if we need one and don't already have one.  */
 
-  /* First try to decide what to do without reference to look-ahead token.  */
+  /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
   if (yyn == YYPACT_NINF)
     goto yydefault;
 
-  /* Not known => get a look-ahead token if don't already have one.  */
+  /* Not known => get a lookahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1625,20 +1641,16 @@ yybackup:
       goto yyreduce;
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   /* Count tokens shifted since error; after three, turn off error
      status.  */
   if (yyerrstatus)
     yyerrstatus--;
 
-  /* Shift the look-ahead token.  */
+  /* Shift the lookahead token.  */
   YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
 
-  /* Discard the shifted token unless it is eof.  */
-  if (yychar != YYEOF)
-    yychar = YYEMPTY;
+  /* Discard the shifted token.  */
+  yychar = YYEMPTY;
 
   yystate = yyn;
   *++yyvsp = yylval;
@@ -1678,17 +1690,20 @@ yyreduce:
   switch (yyn)
     {
         case 3:
-#line 150 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 148 "ftpcmd.y"
     {
-			if (fromname != NULL)
-				free (fromname);
+			free (fromname);
 			fromname = (char *) 0;
 			restart_point = (off_t) 0;
 		}
     break;
 
   case 5:
-#line 161 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 158 "ftpcmd.y"
     {
 			user((yyvsp[(3) - (4)].s));
 			free((yyvsp[(3) - (4)].s));
@@ -1696,7 +1711,9 @@ yyreduce:
     break;
 
   case 6:
-#line 166 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 163 "ftpcmd.y"
     {
 			pass((yyvsp[(3) - (4)].s));
 			memset ((yyvsp[(3) - (4)].s), 0, strlen ((yyvsp[(3) - (4)].s)));
@@ -1705,7 +1722,9 @@ yyreduce:
     break;
 
   case 7:
-#line 172 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 169 "ftpcmd.y"
     {
 			usedefault = 0;
 			if (pdata >= 0) {
@@ -1718,7 +1737,7 @@ yyreduce:
 					sizeof (data_dest.sin_addr)) == 0 &&
 					ntohs (data_dest.sin_port) >
 					IPPORT_RESERVED) {
-					reply (200, "PORT command sucessful.");
+					reply (200, "PORT command successful.");
 				}
 				else {
 					memset (&data_dest, 0,
@@ -1730,7 +1749,9 @@ yyreduce:
     break;
 
   case 8:
-#line 194 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 191 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (3)].i))
 				passive();
@@ -1738,7 +1759,9 @@ yyreduce:
     break;
 
   case 9:
-#line 199 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 196 "ftpcmd.y"
     {
 			switch (cmd_type) {
 
@@ -1776,7 +1799,9 @@ yyreduce:
     break;
 
   case 10:
-#line 234 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 231 "ftpcmd.y"
     {
 			switch ((yyvsp[(3) - (4)].i)) {
 
@@ -1791,7 +1816,9 @@ yyreduce:
     break;
 
   case 11:
-#line 246 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 243 "ftpcmd.y"
     {
 			switch ((yyvsp[(3) - (4)].i)) {
 
@@ -1806,51 +1833,60 @@ yyreduce:
     break;
 
   case 12:
-#line 258 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 255 "ftpcmd.y"
     {
 			reply(202, "ALLO command ignored.");
 		}
     break;
 
   case 13:
-#line 262 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 259 "ftpcmd.y"
     {
 			reply(202, "ALLO command ignored.");
 		}
     break;
 
   case 14:
-#line 266 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 263 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				retrieve((char *) 0, (yyvsp[(4) - (5)].s));
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 15:
-#line 273 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 269 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				store((yyvsp[(4) - (5)].s), "w", 0);
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 16:
-#line 280 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 275 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				store((yyvsp[(4) - (5)].s), "a", 0);
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 17:
-#line 287 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 281 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (3)].i))
 				send_file_list(".");
@@ -1858,17 +1894,20 @@ yyreduce:
     break;
 
   case 18:
-#line 292 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 286 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				send_file_list((yyvsp[(4) - (5)].s));
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 19:
-#line 299 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 292 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (3)].i))
 				retrieve("/bin/ls -lgA", "");
@@ -1876,44 +1915,51 @@ yyreduce:
     break;
 
   case 20:
-#line 304 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 297 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				retrieve("/bin/ls -lgA %s", (yyvsp[(4) - (5)].s));
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 21:
-#line 311 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 303 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				statfilecmd((yyvsp[(4) - (5)].s));
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 22:
-#line 318 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 309 "ftpcmd.y"
     {
 			statcmd();
 		}
     break;
 
   case 23:
-#line 322 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 313 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				delete((yyvsp[(4) - (5)].s));
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 24:
-#line 329 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 319 "ftpcmd.y"
     {
 		    if ((yyvsp[(2) - (5)].i)) {
 			if (fromname) {
@@ -1929,14 +1975,18 @@ yyreduce:
     break;
 
   case 25:
-#line 342 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 332 "ftpcmd.y"
     {
 			reply(225, "ABOR command successful.");
 		}
     break;
 
   case 26:
-#line 346 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 336 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (3)].i))
 				cwd(cred.homedir);
@@ -1944,24 +1994,29 @@ yyreduce:
     break;
 
   case 27:
-#line 351 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 341 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				cwd((yyvsp[(4) - (5)].s));
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 28:
-#line 358 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 347 "ftpcmd.y"
     {
 			help(cmdtab, (char *) 0);
 		}
     break;
 
   case 29:
-#line 362 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 351 "ftpcmd.y"
     {
 			char *cp = (yyvsp[(3) - (4)].s);
 
@@ -1975,40 +2030,45 @@ yyreduce:
 					help(sitetab, (char *) 0);
 			} else
 				help(cmdtab, (yyvsp[(3) - (4)].s));
-			if ((yyvsp[(3) - (4)].s) != NULL)
-			    free ((yyvsp[(3) - (4)].s));
+		    free ((yyvsp[(3) - (4)].s));
 		}
     break;
 
   case 30:
-#line 379 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 367 "ftpcmd.y"
     {
 			reply(200, "NOOP command successful.");
 		}
     break;
 
   case 31:
-#line 383 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 371 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				makedir((yyvsp[(4) - (5)].s));
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 32:
-#line 390 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 377 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				removedir((yyvsp[(4) - (5)].s));
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 33:
-#line 397 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 383 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (3)].i))
 				pwd();
@@ -2016,7 +2076,9 @@ yyreduce:
     break;
 
   case 34:
-#line 402 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 388 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (3)].i))
 				cwd("..");
@@ -2024,23 +2086,28 @@ yyreduce:
     break;
 
   case 35:
-#line 407 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 393 "ftpcmd.y"
     {
 			help(sitetab, (char *) 0);
 		}
     break;
 
   case 36:
-#line 411 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 397 "ftpcmd.y"
     {
 			help(sitetab, (yyvsp[(5) - (6)].s));
-			if ((yyvsp[(5) - (6)].s) != NULL)
-			    free ((yyvsp[(5) - (6)].s));
+			free ((yyvsp[(5) - (6)].s));
 		}
     break;
 
   case 37:
-#line 417 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 402 "ftpcmd.y"
     {
 			int oldmask;
 
@@ -2053,7 +2120,9 @@ yyreduce:
     break;
 
   case 38:
-#line 427 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 412 "ftpcmd.y"
     {
 			int oldmask;
 
@@ -2071,7 +2140,9 @@ yyreduce:
     break;
 
   case 39:
-#line 442 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 427 "ftpcmd.y"
     {
 			if ((yyvsp[(4) - (9)].i) && ((yyvsp[(8) - (9)].s) != NULL)) {
 				if ((yyvsp[(6) - (9)].i) > 0777)
@@ -2082,13 +2153,14 @@ yyreduce:
 				else
 					reply(200, "CHMOD command successful.");
 			}
-			if ((yyvsp[(8) - (9)].s) != NULL)
-				free((yyvsp[(8) - (9)].s));
+			free((yyvsp[(8) - (9)].s));
 		}
     break;
 
   case 40:
-#line 456 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 440 "ftpcmd.y"
     {
 			reply(200,
 			    "Current IDLE time limit is %d seconds; max %d",
@@ -2097,7 +2169,9 @@ yyreduce:
     break;
 
   case 41:
-#line 462 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 446 "ftpcmd.y"
     {
 		    	if ((yyvsp[(3) - (7)].i)) {
 			    if ((yyvsp[(6) - (7)].i) < 30 || (yyvsp[(6) - (7)].i) > maxtimeout) {
@@ -2116,17 +2190,20 @@ yyreduce:
     break;
 
   case 42:
-#line 478 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 462 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				store((yyvsp[(4) - (5)].s), "w", 1);
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 43:
-#line 485 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 468 "ftpcmd.y"
     {
 		        const char *sys_type; /* Official rfc-defined os type.  */
 			char *version = 0; /* A more specific type. */
@@ -2160,24 +2237,26 @@ yyreduce:
 				reply(215, "%s Type: L%d", sys_type, NBBY);
 
 #ifdef HAVE_UNAME
-			if (version)
-				free (version);
+			free (version);
 #endif
 		}
     break;
 
   case 44:
-#line 531 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 513 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL)
 				sizecmd((yyvsp[(4) - (5)].s));
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 45:
-#line 548 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 529 "ftpcmd.y"
     {
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s) != NULL) {
 				struct stat stbuf;
@@ -2196,13 +2275,14 @@ yyreduce:
 					    t->tm_sec);
 				}
 			}
-			if ((yyvsp[(4) - (5)].s) != NULL)
-				free((yyvsp[(4) - (5)].s));
+			free((yyvsp[(4) - (5)].s));
 		}
     break;
 
   case 46:
-#line 570 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 550 "ftpcmd.y"
     {
 			reply(221, "Goodbye.");
 			dologout(0);
@@ -2210,19 +2290,22 @@ yyreduce:
     break;
 
   case 47:
-#line 575 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 555 "ftpcmd.y"
     {
 			yyerrok;
 		}
     break;
 
   case 48:
-#line 581 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 561 "ftpcmd.y"
     {
 			restart_point = (off_t) 0;
 			if ((yyvsp[(2) - (5)].i) && (yyvsp[(4) - (5)].s)) {
-			    if (fromname != NULL)
-				free (fromname);
+			    free (fromname);
 			    fromname = renamefrom((yyvsp[(4) - (5)].s));
 			}
 			if (fromname == (char *) 0 && (yyvsp[(4) - (5)].s))
@@ -2231,10 +2314,11 @@ yyreduce:
     break;
 
   case 49:
-#line 592 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 571 "ftpcmd.y"
     {
-		    	if (fromname != NULL)
-				free (fromname);
+		        free (fromname);
 			fromname = (char *) 0;
 			restart_point = (yyvsp[(3) - (4)].i);	/* XXX $3 is only "int" */
 			reply(350,
@@ -2246,14 +2330,18 @@ yyreduce:
     break;
 
   case 51:
-#line 611 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 589 "ftpcmd.y"
     {
 			(yyval.s) = (char *)calloc(1, sizeof(char));
 		}
     break;
 
   case 54:
-#line 624 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 602 "ftpcmd.y"
     {
 			char *a, *p;
 
@@ -2266,28 +2354,36 @@ yyreduce:
     break;
 
   case 55:
-#line 637 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 615 "ftpcmd.y"
     {
 			(yyval.i) = FORM_N;
 		}
     break;
 
   case 56:
-#line 641 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 619 "ftpcmd.y"
     {
 			(yyval.i) = FORM_T;
 		}
     break;
 
   case 57:
-#line 645 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 623 "ftpcmd.y"
     {
 			(yyval.i) = FORM_C;
 		}
     break;
 
   case 58:
-#line 652 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 630 "ftpcmd.y"
     {
 			cmd_type = TYPE_A;
 			cmd_form = FORM_N;
@@ -2295,7 +2391,9 @@ yyreduce:
     break;
 
   case 59:
-#line 657 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 635 "ftpcmd.y"
     {
 			cmd_type = TYPE_A;
 			cmd_form = (yyvsp[(3) - (3)].i);
@@ -2303,7 +2401,9 @@ yyreduce:
     break;
 
   case 60:
-#line 662 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 640 "ftpcmd.y"
     {
 			cmd_type = TYPE_E;
 			cmd_form = FORM_N;
@@ -2311,7 +2411,9 @@ yyreduce:
     break;
 
   case 61:
-#line 667 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 645 "ftpcmd.y"
     {
 			cmd_type = TYPE_E;
 			cmd_form = (yyvsp[(3) - (3)].i);
@@ -2319,14 +2421,18 @@ yyreduce:
     break;
 
   case 62:
-#line 672 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 650 "ftpcmd.y"
     {
 			cmd_type = TYPE_I;
 		}
     break;
 
   case 63:
-#line 676 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 654 "ftpcmd.y"
     {
 			cmd_type = TYPE_L;
 			cmd_bytesz = NBBY;
@@ -2334,7 +2440,9 @@ yyreduce:
     break;
 
   case 64:
-#line 681 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 659 "ftpcmd.y"
     {
 			cmd_type = TYPE_L;
 			cmd_bytesz = (yyvsp[(3) - (3)].i);
@@ -2342,7 +2450,9 @@ yyreduce:
     break;
 
   case 65:
-#line 687 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 665 "ftpcmd.y"
     {
 			cmd_type = TYPE_L;
 			cmd_bytesz = (yyvsp[(2) - (2)].i);
@@ -2350,49 +2460,63 @@ yyreduce:
     break;
 
   case 66:
-#line 695 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 673 "ftpcmd.y"
     {
 			(yyval.i) = STRU_F;
 		}
     break;
 
   case 67:
-#line 699 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 677 "ftpcmd.y"
     {
 			(yyval.i) = STRU_R;
 		}
     break;
 
   case 68:
-#line 703 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 681 "ftpcmd.y"
     {
 			(yyval.i) = STRU_P;
 		}
     break;
 
   case 69:
-#line 710 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 688 "ftpcmd.y"
     {
 			(yyval.i) = MODE_S;
 		}
     break;
 
   case 70:
-#line 714 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 692 "ftpcmd.y"
     {
 			(yyval.i) = MODE_B;
 		}
     break;
 
   case 71:
-#line 718 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 696 "ftpcmd.y"
     {
 			(yyval.i) = MODE_C;
 		}
     break;
 
   case 72:
-#line 725 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 703 "ftpcmd.y"
     {
 			/*
 			 * Problem: this production is used for all pathname
@@ -2429,7 +2553,9 @@ yyreduce:
     break;
 
   case 74:
-#line 766 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 744 "ftpcmd.y"
     {
 			int ret, dec, multby, digit;
 
@@ -2455,7 +2581,9 @@ yyreduce:
     break;
 
   case 75:
-#line 793 "ftpcmd.y"
+
+/* Line 1455 of yacc.c  */
+#line 771 "ftpcmd.y"
     {
 			if (cred.logged_in)
 				(yyval.i) = 1;
@@ -2467,8 +2595,9 @@ yyreduce:
     break;
 
 
-/* Line 1267 of yacc.c.  */
-#line 2472 "ftpcmd.c"
+
+/* Line 1455 of yacc.c  */
+#line 2601 "ftpcmd.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2479,7 +2608,6 @@ yyreduce:
 
   *++yyvsp = yyval;
 
-
   /* Now `shift' the result of the reduction.  Determine what state
      that goes to, based on the state we popped back to and the rule
      number reduced by.  */
@@ -2544,7 +2672,7 @@ yyerrlab:
 
   if (yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse look-ahead token after an
+      /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
       if (yychar <= YYEOF)
@@ -2561,7 +2689,7 @@ yyerrlab:
 	}
     }
 
-  /* Else will try to reuse look-ahead token after shifting the error
+  /* Else will try to reuse lookahead token after shifting the error
      token.  */
   goto yyerrlab1;
 
@@ -2618,9 +2746,6 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   *++yyvsp = yylval;
 
 
@@ -2645,7 +2770,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
-#ifndef yyoverflow
+#if !defined(yyoverflow) || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -2656,7 +2781,7 @@ yyexhaustedlab:
 #endif
 
 yyreturn:
-  if (yychar != YYEOF && yychar != YYEMPTY)
+  if (yychar != YYEMPTY)
      yydestruct ("Cleanup: discarding lookahead",
 		 yytoken, &yylval);
   /* Do not reclaim the symbols of the rule which action triggered
@@ -2682,7 +2807,9 @@ yyreturn:
 }
 
 
-#line 803 "ftpcmd.y"
+
+/* Line 1675 of yacc.c  */
+#line 781 "ftpcmd.y"
 
 
 #define	CMD	0	/* beginning of command */
