@@ -1,3 +1,23 @@
+/*
+  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
+  2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation,
+  Inc.
+
+  This file is part of GNU Inetutils.
+
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
+
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
+
 /* - Ftp Server
  * Copyright (c) 1985, 1988, 1990, 1992, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
@@ -10,7 +30,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -27,26 +47,6 @@
  * SUCH DAMAGE.
  */
 
-/* Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
-   Free Software Foundation, Inc.
-
-   This file is part of GNU Inetutils.
-
-   GNU Inetutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
-
-   GNU Inetutils is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write
-   to the Free Software Foundation, Inc., 51 Franklin Street,
-   Fifth Floor, Boston, MA 02110-1301 USA. */
-
 /*
  * FTP server.
  */
@@ -55,23 +55,7 @@
 # include <config.h>
 #endif
 
-#if !defined (__GNUC__) && defined (_AIX)
-# pragma alloca
-#endif
-#ifndef alloca			/* Make alloca work the best possible way.  */
-# ifdef __GNUC__
-#  define alloca __builtin_alloca
-# else /* not __GNUC__ */
-#  if HAVE_ALLOCA_H
-#   include <alloca.h>
-#  else	/* not __GNUC__ or HAVE_ALLOCA_H */
-#   ifndef _AIX			/* Already did AIX, up at the top.  */
-char *alloca ();
-#   endif /* not _AIX */
-#  endif /* not HAVE_ALLOCA_H */
-# endif	/* not __GNUC__ */
-#endif /* not alloca */
-
+#include <alloca.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -97,17 +81,12 @@ char *alloca ();
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <getopt.h>
 #include <limits.h>
 #include <netdb.h>
 #include <setjmp.h>
 #include <signal.h>
 #include <grp.h>
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-# include <stdarg.h>
-#else
-# include <varargs.h>
-#endif
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -129,6 +108,8 @@ char *alloca ();
 /* Include glob.h last, because it may define "const" which breaks
    system headers on some platforms. */
 #include <glob.h>
+#include <argp.h>
+#include <error.h>
 
 #include <progname.h>
 #include <libinetutils.h>
@@ -273,77 +254,170 @@ static void myoob (int);
 static int receive_data (FILE *, FILE *);
 static void send_data (FILE *, FILE *, off_t);
 static void sigquit (int);
-static void usage (int);
 
-static const char *short_options = "Aa:Ddlp:qt:T:u:";
-static struct option long_options[] = {
-  {"anonymous-only", no_argument, 0, 'A'},
-  {"auth", required_argument, 0, 'a'},
-  {"daemon", no_argument, 0, 'D'},
-  {"debug", no_argument, 0, 'd'},
-  {"help", no_argument, 0, '&'},
-  {"logging", no_argument, 0, 'l'},
-  {"pidfile", required_argument, 0, 'p'},
-  {"no-version", no_argument, 0, 'q'},
-  {"timeout", required_argument, 0, 't'},
-  {"max-timeout", required_argument, 0, 'T'},
-  {"umask", required_argument, 0, 'u'},
-  {"version", no_argument, 0, 'V'},
-  {0, 0, 0, 0}
-};
+const char doc[] = "File Transfer Protocol Daemon";
 
-static void
-usage (int err)
-{
-  if (err != 0)
-    {
-      fprintf (stderr, "Usage: %s [OPTION] ...\n", program_name);
-      fprintf (stderr, "Try `%s --help' for more information.\n",
-	       program_name);
-    }
-  else
-    {
-      fprintf (stdout, "Usage: %s [OPTION] ...\n", program_name);
-      puts ("Internet File Transfer Protocol server.\n\n\
-  -A, --anonymous-only      Server configure for anonymous service only\n\
-  -D, --daemon              Start the ftpd standalone\n\
-  -d, --debug               Debug mode\n\
-  -l, --logging             Increase verbosity of syslog messages\n\
-  -p, --pidfile=[PIDFILE]   Change default location of pidfile\n\
-  -q, --no-version          Do not display version in banner\n\
-  -t, --timeout=[TIMEOUT]   Set default idle timeout\n\
-  -T, --max-timeout         Reset maximum value of timeout allowed\n\
-  -u, --umask               Set default umask(base 8)\n\
-      --help                Print this message\n\
-  -V, --version             Print version\n\
-  -a, --auth=[AUTH]         Use AUTH for authentication, it can be:\n\
-                               default     passwd authentication.");
+static struct argp_option options[] = {
+#define GRID 0
+  { "anonymous-only", 'A', NULL, 0,
+    "server configured for anonymous service only",
+    GRID+1 },
+  { "daemon", 'D', NULL, 0,
+    "start the ftpd standalone",
+    GRID+1 },
+  { "debug", 'd', NULL, 0,
+    "debug mode",
+    GRID+1 },
+  { "logging", 'l', NULL, 0,
+    "increase verbosity of syslog messages",
+    GRID+1 },
+  { "pidfile", 'p', "PIDFILE", OPTION_ARG_OPTIONAL,
+    "change default location of pidfile",
+    GRID+1 },
+  { "no-version", 'q', NULL, 0,
+    "do not display version in banner",
+    GRID+1 },
+  { "timeout", 't', "TIMEOUT", 0,
+    "set default idle timeout",
+    GRID+1 },
+  { "max-timeout", 'T', NULL, 0,
+    "reset maximum value of timeout allowed",
+    GRID+1 },
+  { "umask", 'u', "VAL", 0,
+    "set default umask",
+    GRID+1 },
+  { "auth", 'a', "AUTH", OPTION_ARG_OPTIONAL,
+    "use AUTH for authentication",
+    GRID+1 },
+  { NULL, 0, NULL, 0, "AUTH can be one of the following:", GRID+2 },
+  { "  default", 0, NULL, OPTION_DOC|OPTION_NO_TRANS,
+    "passwd authentication",
+    GRID+3 },
 #ifdef WITH_PAM
-      puts ("\
-                               pam         using pam 'ftp' module.");
+  { "  pam", 0, NULL, OPTION_DOC|OPTION_NO_TRANS,
+    "using pam 'ftp' module",
+    GRID+3 },
 #endif
 #ifdef WITH_KERBEROS
-      puts ("\
-                               kerberos");
+  { "  kerberos", 0, NULL, OPTION_DOC|OPTION_NO_TRANS,
+    "",
+    GRID+3 },
 #endif
 #ifdef WITH_KERBEROS5
-      puts ("\
-                               kderberos5");
+  { "  kderberos5", 0, NULL, OPTION_DOC|OPTION_NO_TRANS,
+    "",
+    GRID+3 },
 #endif
 #ifdef WITH_OPIE
-      puts ("\
-                               opie");
+  { "  opie", 0, NULL, OPTION_DOC|OPTION_NO_TRANS,
+    "",
+    GRID+3 },
 #endif
+  { NULL }
+};
 
-      fprintf (stdout, "\nSubmit bug reports to %s.\n", PACKAGE_BUGREPORT);
+static error_t
+parse_opt (int key, char *arg, struct argp_state *state)
+{
+  switch (key)
+    {
+    case 'A':
+      /* Anonymous ftp only.  */
+      anon_only = 1;
+      break;
+
+    case 'a':
+      if (strcasecmp (arg, "default") == 0)
+	cred.auth_type = AUTH_TYPE_PASSWD;
+#ifdef WITH_PAM
+      else if (strcasecmp (arg, "pam") == 0)
+	cred.auth_type = AUTH_TYPE_PAM;
+#endif
+#ifdef WITH_KERBEROS
+      else if (strcasecmp (arg, "kerberos") == 0)
+	cred.auth_type = AUTH_TYPE_KERBEROS;
+#endif
+#ifdef WITH_KERBEROS5
+      else if (strcasecmp (arg, "kerberos5") == 0)
+	cred.auth_type = AUTH_TYPE_KERBEROS5;
+#endif
+#ifdef WITH_OPIE
+      else if (strcasecmp (arg, "opie") == 0)
+	cred.auth_type = AUTH_TYPE_OPIE;
+#endif
+      break;
+
+    case 'D':
+      /* Run ftpd as daemon.  */
+      daemon_mode = 1;
+      break;
+
+    case 'd':
+      /* Enable debug mode.  */
+      debug = 1;
+      break;
+
+    case 'l':
+      /* Increase logging level.  */
+      logging++;		/* > 1 == Extra logging.  */
+      break;
+
+    case 'p':
+      /* Override pid file */
+      pid_file = arg;
+      break;
+
+    case 'q':
+      /* Don't include version number in banner.  */
+      no_version = 1;
+      break;
+
+    case 't':
+      /* Set default timeout value.  */
+      timeout = atoi (arg);
+      if (maxtimeout < timeout)
+	maxtimeout = timeout;
+      break;
+
+    case 'T':		/* Maximum timeout allowed.  */
+      maxtimeout = atoi (arg);
+      if (timeout > maxtimeout)
+	timeout = maxtimeout;
+      break;
+      
+    case 'u':		/* Set umask.  */
+      {
+	long val = 0;
+
+	val = strtol (arg, &arg, 8);
+	if (*arg != '\0' || val < 0)
+	  argp_error (state, "bad value for -u");
+	else
+	  defumask = val;
+	break;
+      }
+      
+    default:
+      return ARGP_ERR_UNKNOWN;
     }
-  exit (err);
+
+  return 0;
 }
 
+static struct argp argp = {
+  options,
+  parse_opt,
+  NULL,
+  doc,
+  NULL,
+  NULL,
+  NULL
+};
+
 int
 main (int argc, char *argv[], char **envp)
 {
-  int option;
+  int index;
 
   set_program_name (argv[0]);
 
@@ -356,99 +430,15 @@ main (int argc, char *argv[], char **envp)
   initsetproctitle (argc, argv, envp);
 #endif /* HAVE_INITSETPROCTITLE */
 
-  while ((option = getopt_long (argc, argv, short_options,
-				long_options, NULL)) != EOF)
-    {
-      switch (option)
-	{
-	case 'A':		/* Anonymous ftp only.  */
-	  anon_only = 1;
-	  break;
-
-	case 'a':		/* Authentification method.  */
-	  if (strcasecmp (optarg, "default") == 0)
-	    cred.auth_type = AUTH_TYPE_PASSWD;
-#ifdef WITH_PAM
-	  else if (strcasecmp (optarg, "pam") == 0)
-	    cred.auth_type = AUTH_TYPE_PAM;
-#endif
-#ifdef WITH_KERBEROS
-	  else if (stracasecmp (optarg, "kerberos") == 0)
-	    cred.auth_type = AUTH_TYPE_KERBEROS;
-#endif
-#ifdef WITH_KERBEROS5
-	  else if (stracasecmp (optarg, "kerberos5") == 0)
-	    cred.auth_type = AUTH_TYPE_KERBEROS5;
-#endif
-#ifdef WITH_OPIE
-	  else if (stracasecmp (optarg, "opie") == 0)
-	    cred.auth_type = AUTH_TYPE_OPIE;
-#endif
-	  break;
-
-	case 'D':		/* Run ftpd as daemon.  */
-	  daemon_mode = 1;
-	  break;
-
-	case 'd':		/* Enable debug mode.  */
-	  debug = 1;
-	  break;
-
-	case 'l':		/* Increase logging level.  */
-	  logging++;		/* > 1 == Extra logging.  */
-	  break;
-
-	case 'p':		/* Override pid file */
-	  pid_file = optarg;
-	  break;
-
-	case 'q':		/* Don't include version number in banner.  */
-	  no_version = 1;
-	  break;
-
-	case 't':		/* Set default timeout value.  */
-	  timeout = atoi (optarg);
-	  if (maxtimeout < timeout)
-	    maxtimeout = timeout;
-	  break;
-
-	case 'T':		/* Maximum timeout allowed.  */
-	  maxtimeout = atoi (optarg);
-	  if (timeout > maxtimeout)
-	    timeout = maxtimeout;
-	  break;
-
-	case 'u':		/* Set umask.  */
-	  {
-	    long val = 0;
-
-	    val = strtol (optarg, &optarg, 8);
-	    if (*optarg != '\0' || val < 0)
-	      fprintf (stderr, "%s: bad value for -u", argv[0]);
-	    else
-	      defumask = val;
-	    break;
-	  }
-
-	case '&':		/* Usage.  */
-	  usage (0);
-	  /* Not reached.  */
-
-	case 'V':		/* Version.  */
-	  printf ("ftpd (%s) %s\n", PACKAGE_NAME, PACKAGE_VERSION);
-	  exit (0);
-
-	case '?':
-	default:
-	  usage (1);
-	  /* Not reached.  */
-	}
-    }
-
+  /* Parse the command line */
+  iu_argp_init ("ftpd", default_program_authors);
+  argp_parse (&argp, argc, argv, 0, &index, NULL);
+  
   /* Bail out, wrong usage */
-  argc -= optind;
+  argc -= index;
   if (argc != 0)
-    usage (1);
+    error (1, 0, "surplus arguments; try `%s --help' for more info",
+	   program_name);
 
   /* LOG_NDELAY sets up the logging connection immediately,
      necessary for anonymous ftp's that chroot and can't do it later.  */
@@ -566,8 +556,7 @@ curdir (void)
 {
   static char *path = 0;
   extern char *xgetcwd (void);
-  if (path)
-    free (path);
+  free (path);
   path = xgetcwd ();
   if (!path)
     return (char *) "";
@@ -790,26 +779,21 @@ end_login (struct credentials *pcred)
   if (pcred->logged_in)
     logwtmp_keep_open (ttyline, "", "");
 
-  if (pcred->name)
-    free (pcred->name);
+  free (pcred->name);
   if (pcred->passwd)
     {
       memset (pcred->passwd, 0, strlen (pcred->passwd));
       free (pcred->passwd);
     }
-  if (pcred->homedir)
-    free (pcred->homedir);
-  if (pcred->rootdir)
-    free (pcred->rootdir);
-  if (pcred->shell)
-    free (pcred->shell);
+  free (pcred->homedir);
+  free (pcred->rootdir);
+  free (pcred->shell);
   if (pcred->pass)		/* ??? */
     {
       memset (pcred->pass, 0, strlen (pcred->pass));
       free (pcred->pass);
     }
-  if (pcred->message)
-    free (pcred->message);
+  free (pcred->message);
   memset (pcred, 0, sizeof (*pcred));
   pcred->remotehost = remotehost;
   pcred->auth_type = atype;
@@ -1676,8 +1660,7 @@ dolog (struct sockaddr_in *sin, struct credentials *pcred)
   else
     name = inet_ntoa (sin->sin_addr);
 
-  if (pcred->remotehost)
-    free (pcred->remotehost);
+  free (pcred->remotehost);
   pcred->remotehost = sgetsave (name);
 
 #ifdef HAVE_SETPROCTITLE
@@ -1809,9 +1792,7 @@ gunique (const char *local)
   if (cp)
     *cp = '/';
 
-  if (string)
-    free (string);
-
+  free (string);
   string = malloc (strlen (local) + 5);	/* '.' + DIG + DIG + '\0' */
   if (string)
     {
@@ -1951,8 +1932,7 @@ send_file_list (const char *whichf)
 	      dir->d_name[2] == '\0')
 	    continue;
 
-	  nbuf = (char *) alloca (strlen (dirname) + 1 +
-				  strlen (dir->d_name) + 1);
+	  nbuf = alloca (strlen (dirname) + 1 + strlen (dir->d_name) + 1);
 	  sprintf (nbuf, "%s/%s", dirname, dir->d_name);
 
 	  /* We have to do a stat to insure it's
@@ -1990,8 +1970,7 @@ send_file_list (const char *whichf)
   data = -1;
   pdata = -1;
 out:
-  if (p)
-    free (p);
+  free (p);
   if (freeglob)
     {
       freeglob = 0;
