@@ -1,24 +1,23 @@
 /* linux.c -- Linux specific code for ifconfig
+  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
+  Free Software Foundation, Inc.
 
-   Copyright (C) 2001, 2002, 2005, 2007 Free Software Foundation, Inc.
+  This file is part of GNU Inetutils.
 
-   Written by Marcus Brinkmann.
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 3
-   of the License, or (at your option) any later version.
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA.
- */
+/* Written by Marcus Brinkmann.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -27,6 +26,7 @@
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
+#include <xalloc.h>
 
 #if HAVE_UNISTD_H
 # include <unistd.h>
@@ -274,12 +274,256 @@ arphrd_findvalue (int value)
   else
     return NULL;
 }
+
+
+struct pnd_stats
+{
+  struct pnd_stats *next;
+  char *name;
+  unsigned long long rx_packets;   /* total packets received */
+  unsigned long long tx_packets;   /* total packets transmitted */
+  unsigned long long rx_bytes;     /* total bytes received */
+  unsigned long long tx_bytes;     /* total bytes transmitted */
+  unsigned long rx_errors;         /* packets receive errors */
+  unsigned long tx_errors;         /* packet transmit errors */
+  unsigned long rx_dropped;        /* input packets dropped */
+  unsigned long tx_dropped;        /* transmit packets dropped */
+  unsigned long rx_multicast;      /* multicast packets received */
+  unsigned long rx_compressed;     /* compressed packets received */ 
+  unsigned long tx_compressed;     /* compressed packets transmitted */
+  unsigned long collisions;
+
+  /* detailed rx_errors: */
+  unsigned long rx_length_errors;
+   unsigned long rx_over_errors;    /* receiver ring buffer overflow */
+  unsigned long rx_crc_errors;     /* received packets with crc error */
+  unsigned long rx_frame_errors;   /* received frame alignment errors */
+  unsigned long rx_fifo_errors;    /* recveiver fifo overruns */
+  unsigned long rx_missed_errors;  /* receiver missed packets */
+  /* detailed tx_errors */
+  unsigned long tx_aborted_errors;
+  unsigned long tx_carrier_errors;
+  unsigned long tx_fifo_errors;
+  unsigned long tx_heartbeat_errors;
+  unsigned long tx_window_errors;
+};
+
+struct pnd_stats *stats_head, *stats_tail;
+static int pnd_stats_init = 0;
+
+static int
+procnet_parse_fields_v1 (char *buf, struct pnd_stats *stats)
+{
+  int n = sscanf (buf,
+		  "%llu %lu %lu %lu %lu %llu %lu %lu %lu %lu %lu",
+		  &stats->rx_packets,
+		  &stats->rx_errors,
+		  &stats->rx_dropped,
+		  &stats->rx_fifo_errors,
+		  &stats->rx_frame_errors,
+		  &stats->tx_packets,
+		  &stats->tx_errors,
+		  &stats->tx_dropped,
+		  &stats->tx_fifo_errors,
+		  &stats->collisions,
+		  &stats->tx_carrier_errors);
+  stats->rx_bytes = 0;
+  stats->tx_bytes = 0;
+  stats->rx_multicast = 0;
+  return n == 11;
+}
+
+static int
+procnet_parse_fields_v2 (char *buf, struct pnd_stats *stats)
+{
+  int n = sscanf (buf,
+		  "%llu %llu %lu %lu %lu %lu %llu %llu %lu %lu %lu %lu %lu",
+		  &stats->rx_bytes,
+		  &stats->rx_packets,
+		  &stats->rx_errors,
+		  &stats->rx_dropped,
+		  &stats->rx_fifo_errors,
+		  &stats->rx_frame_errors,
+		  &stats->tx_bytes,
+		  &stats->tx_packets,
+		  &stats->tx_errors,
+		  &stats->tx_dropped,
+		  &stats->tx_fifo_errors,
+		  &stats->collisions,
+		  &stats->tx_carrier_errors);
+  stats->rx_multicast = 0;
+  return n == 13;
+}
+
+static int
+procnet_parse_fields_v3 (char *buf, struct pnd_stats *stats)
+{
+  int n = sscanf (buf,
+		  "%llu %llu %lu %lu %lu %lu %lu %lu %llu %llu %lu %lu %lu %lu %lu %lu",
+		  &stats->rx_bytes,
+		  &stats->rx_packets,
+		  &stats->rx_errors,
+		  &stats->rx_dropped,
+		  &stats->rx_fifo_errors,
+		  &stats->rx_frame_errors,
+		  &stats->rx_compressed,
+		  &stats->rx_multicast,
+		  &stats->tx_bytes,
+		  &stats->tx_packets,
+		  &stats->tx_errors,
+		  &stats->tx_dropped,
+		  &stats->tx_fifo_errors,
+		  &stats->collisions,
+		  &stats->tx_carrier_errors,
+		  &stats->tx_compressed);
+  return n == 16;
+}
+
+static int (*pnd_parser[]) (char *, struct pnd_stats *) =
+{
+  procnet_parse_fields_v1,
+  procnet_parse_fields_v2,
+  procnet_parse_fields_v3
+};
+
+static int
+pnd_version (char *buf)
+{
+  if (strstr (buf, "compressed"))
+    return 2;
+  if (strstr(buf, "bytes"))
+    return 1;
+  return 0;
+}
+
+static void
+pnd_read ()
+{
+  FILE *fp;
+  char *buf = NULL;
+  size_t bufsize = 0;
+  int v;
+
+  fp = fopen (PATH_PROCNET_DEV, "r");
+  if (!fp)
+    {
+      error (0, errno, "cannot open %s", PATH_PROCNET_DEV);
+      return;
+    }
+
+  /* Skip header lines */
+  if (getline (&buf, &bufsize, fp) < 0
+      || getline (&buf, &bufsize, fp) < 0)
+    {
+      error (0, errno, "malformed %s", PATH_PROCNET_DEV);
+      fclose (fp);
+      free (buf);
+      return;
+    }
+
+  v = pnd_version (buf);
+
+  while (getline (&buf, &bufsize, fp) > 0)
+    {
+      struct pnd_stats *stats = xzalloc (sizeof (*stats));;
+      char *p = buf, *q;
+      size_t len;
+      
+      while (*p
+	     && isascii (*(unsigned char*) p) && isspace (*(unsigned char*) p))
+	p++;
+      q = strchr (p, ':');
+      if (!q)
+	{
+	  error (0, errno, "malformed %s", PATH_PROCNET_DEV);
+	  free (stats);
+	  break;
+	}
+      len = q - p;
+      stats->name = xmalloc (len + 1);
+      memcpy (stats->name, p, len);
+      stats->name[len] = 0;
+      if (pnd_parser[v] (q + 1, stats) == 0)
+	{
+	  error (0, errno, "malformed %s", PATH_PROCNET_DEV);
+	  free (stats);
+	  break;
+	}
+      stats->next = NULL;
+      if (stats_tail)
+	stats_tail->next = stats;
+      else
+	stats_head = stats;
+      stats_tail = stats;
+    }
+
+  fclose (fp);
+  free (buf);
+}
+
+struct pnd_stats *
+pnd_stats_locate (const char *name)
+{
+  struct pnd_stats *stats;
+  if (!pnd_stats_init)
+    {
+      pnd_read ();
+      pnd_stats_init = 1;
+    }
+  for (stats = stats_head; stats; stats = stats->next)
+    if (strcmp (stats->name, name) == 0)
+      break;
+  return stats;
+}
+
 
 
 /* Output format stuff.  */
 
 const char *system_default_format = "net-tools";
 
+void
+system_fh_ifstat_query (format_data_t form, int argc, char *argv[])
+{
+  select_arg (form, argc, argv,
+	      pnd_stats_locate (form->ifr->ifr_name) ? 0 : 1);
+}  
+
+#define _IU_DECLARE(fld)						     \
+void                                                                         \
+_IU_CAT2 (system_fh_,fld) (format_data_t form, int argc, char *argv[])       \
+{                                                                            \
+  struct pnd_stats *stats = pnd_stats_locate (form->ifr->ifr_name);          \
+  if (!stats)                                                                \
+    put_string (form, "(" #fld " unknown)");                                 \
+  else 								             \
+    put_ulong (form, argc, argv, stats->fld);                                \
+}
+
+_IU_DECLARE (rx_packets)
+_IU_DECLARE (tx_packets)
+_IU_DECLARE (rx_bytes)
+_IU_DECLARE (tx_bytes)
+_IU_DECLARE (rx_errors)
+_IU_DECLARE (tx_errors)
+_IU_DECLARE (rx_dropped)
+_IU_DECLARE (tx_dropped)
+_IU_DECLARE (rx_multicast)
+_IU_DECLARE (rx_compressed)
+_IU_DECLARE (tx_compressed)
+_IU_DECLARE (collisions)
+_IU_DECLARE (rx_length_errors)
+_IU_DECLARE (rx_over_errors)
+_IU_DECLARE (rx_crc_errors)
+_IU_DECLARE (rx_frame_errors)
+_IU_DECLARE (rx_fifo_errors)
+_IU_DECLARE (rx_missed_errors)
+_IU_DECLARE (tx_aborted_errors)
+_IU_DECLARE (tx_carrier_errors)
+_IU_DECLARE (tx_fifo_errors)
+_IU_DECLARE (tx_heartbeat_errors)
+_IU_DECLARE (tx_window_errors)
+
 void
 system_fh_hwaddr_query (format_data_t form, int argc, char *argv[])
 {
@@ -301,11 +545,9 @@ system_fh_hwaddr (format_data_t form, int argc, char *argv[])
 {
 #ifdef SIOCGIFHWADDR
   if (ioctl (form->sfd, SIOCGIFHWADDR, form->ifr) < 0)
-    {
-      fprintf (stderr, "%s: SIOCGIFHWADDR failed for interface `%s': %s\n",
-	       program_name, form->ifr->ifr_name, strerror (errno));
-      exit (EXIT_FAILURE);
-    }
+    error (EXIT_FAILURE, errno,
+           "SIOCGIFHWADDR failed for interface `%s'",
+	   form->ifr->ifr_name);
   else
     {
       struct arphrd_symbol *arp;
@@ -338,11 +580,9 @@ system_fh_hwtype (format_data_t form, int argc, char *argv[])
 {
 #ifdef SIOCGIFHWADDR
   if (ioctl (form->sfd, SIOCGIFHWADDR, form->ifr) < 0)
-    {
-      fprintf (stderr, "%s: SIOCGIFHWADDR failed for interface `%s': %s\n",
-	       program_name, form->ifr->ifr_name, strerror (errno));
-      exit (EXIT_FAILURE);
-    }
+    error (EXIT_FAILURE, errno,
+           "SIOCGIFHWADDR failed for interface `%s'",
+	   form->ifr->ifr_name);
   else
     {
       struct arphrd_symbol *arp;
@@ -375,11 +615,9 @@ system_fh_txqlen (format_data_t form, int argc, char *argv[])
 {
 #ifdef SIOCGIFTXQLEN
   if (ioctl (form->sfd, SIOCGIFTXQLEN, form->ifr) < 0)
-    {
-      fprintf (stderr, "%s: SIOCGIFTXQLEN failed for interface `%s': %s\n",
-	       program_name, form->ifr->ifr_name, strerror (errno));
-      exit (EXIT_FAILURE);
-    }
+    error (EXIT_FAILURE, errno,
+           "SIOCGIFTXQLEN failed for interface `%s'",
+	   form->ifr->ifr_name);
   else
     put_int (form, argc, argv, form->ifr->ifr_qlen);
 #else
@@ -392,12 +630,9 @@ system_fh_txqlen (format_data_t form, int argc, char *argv[])
 /* Argument parsing stuff.  */
 
 const char *system_help = "\
-  NAME [ADDR] [broadcast BRDADDR]\n\
-  [pointopoint|dstaddr DSTADDR] [netmask MASK]\n\
-  [metric N] [mtu N] [txqueuelen N]";
-
-const char *system_help_options =
-  "      --txqlen N        Set transmit queue length to N";
+ NAME [ADDR] [broadcast BRDADDR]\
+ [pointopoint|dstaddr DSTADDR] [netmask MASK]\
+ [metric N] [mtu N] [txqueuelen N]";
 
 void
 system_parse_opt_set_txqlen (struct ifconfig *ifp, char *arg)
@@ -405,40 +640,62 @@ system_parse_opt_set_txqlen (struct ifconfig *ifp, char *arg)
   char *end;
 
   if (!ifp)
-    {
-      fprintf (stderr, "%s: no interface specified for txqlen"
-	       " `%s'\n", program_name, arg);
-      usage (EXIT_FAILURE);
-    }
+    error (EXIT_FAILURE, 0,
+           "no interface specified for txqlen `%s'", arg);
 
   if (!(ifp->valid & IF_VALID_SYSTEM))
     {
       ifp->system = malloc (sizeof (struct system_ifconfig));
       if (!ifp->system)
-	{
-	  fprintf (stderr, "%s: can't get memory for system interface "
-		   "configuration: %s\n", program_name, strerror (errno));
-	  exit (EXIT_FAILURE);
-	}
+	error (EXIT_FAILURE, errno,
+	       "can't get memory for system interface configuration");
       ifp->system->valid = 0;
       ifp->valid |= IF_VALID_SYSTEM;
     }
   if (ifp->system->valid & IF_VALID_TXQLEN)
-    {
-      fprintf (stderr, "%s: only one txqlen allowed for interface `%s'\n",
-	       program_name, ifp->name);
-      usage (EXIT_FAILURE);
-    }
+    error (EXIT_FAILURE, 0,
+           "only one txqlen allowed for interface `%s'",
+	   ifp->name);
   ifp->system->txqlen = strtol (arg, &end, 0);
   if (*arg == '\0' || *end != '\0')
-    {
-      fprintf (stderr, "%s: txqlen value `%s' for interface `%s' "
-	       "is not a number.\n", program_name, arg, ifp->name);
-      exit (EXIT_FAILURE);
-    }
+    error (EXIT_FAILURE, 0,
+           "txqlen value `%s' for interface `%s' is not a number", 
+           arg, ifp->name);
   ifp->system->valid |= IF_VALID_TXQLEN;
 }
 
+static struct argp_option linux_argp_options[] = {
+  { "txqlen", 'T', "N", 0,
+    "set transmit queue length to N" },
+  { NULL }
+};
+
+static error_t
+linux_argp_parser (int key, char *arg, struct argp_state *state)
+{
+  struct ifconfig *ifp = *(struct ifconfig **)state->input;
+  switch (key)
+    {
+    case 'T':			/* txqlen */
+      system_parse_opt_set_txqlen (ifp, arg);
+      break;
+
+    default:
+      return ARGP_ERR_UNKNOWN;
+    }
+  return 0;
+}
+
+static struct argp linux_argp = { linux_argp_options, linux_argp_parser };
+
+struct argp_child system_argp_child = {
+  &linux_argp,
+  0,
+  "Linux-specific options",
+  0
+};
+
+
 int
 system_parse_opt (struct ifconfig **ifpp, char option, char *arg)
 {
@@ -447,7 +704,7 @@ system_parse_opt (struct ifconfig **ifpp, char option, char *arg)
   switch (option)
     {
     case 'T':			/* txqlen */
-      system_parse_opt_set_txqlen (ifp, optarg);
+      system_parse_opt_set_txqlen (ifp, arg);
       break;
 
     default:
@@ -462,6 +719,7 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
   int i = 0;
   enum
   {
+    EXPECT_INET,
     EXPECT_NOTHING,
     EXPECT_BROADCAST,
     EXPECT_DSTADDR,
@@ -469,7 +727,7 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
     EXPECT_MTU,
     EXPECT_METRIC,
     EXPECT_TXQLEN
-  } expect = EXPECT_NOTHING;
+  } expect = EXPECT_INET;
 
   *ifp = parse_opt_new_ifs (argv[0]);
 
@@ -501,6 +759,12 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
 	  system_parse_opt_set_txqlen (*ifp, argv[i]);
 	  break;
 
+	case EXPECT_INET:
+	  expect = EXPECT_NOTHING;
+	  if (!strcmp (argv[i], "inet"))
+	    continue;
+	  break;
+	  
 	case EXPECT_NOTHING:
 	  break;
 	}
@@ -527,43 +791,34 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
   switch (expect)
     {
     case EXPECT_BROADCAST:
-      fprintf (stderr, "%s: option `broadcast' requires an argument\n",
-	       program_name);
+      error (0, 0, "option `broadcast' requires an argument");
       break;
 
     case EXPECT_DSTADDR:
-      fprintf (stderr,
-	       "%s: option `pointopoint' (`dstaddr') requires an argument\n",
-	       program_name);
+      error (0, 0, "option `pointopoint' (`dstaddr') requires an argument");
       break;
 
     case EXPECT_NETMASK:
-      fprintf (stderr, "%s: option `netmask' requires an argument\n",
-	       program_name);
+      error (0, 0, "option `netmask' requires an argument");
       break;
 
     case EXPECT_METRIC:
-      fprintf (stderr, "%s: option `metric' requires an argument\n",
-	       program_name);
+      error (0, 0, "option `metric' requires an argument");
       break;
 
     case EXPECT_MTU:
-      fprintf (stderr, "%s: option `mtu' requires an argument\n",
-	       program_name);
+      error (0, 0, "option `mtu' requires an argument");
       break;
 
     case EXPECT_TXQLEN:
-      fprintf (stderr, "%s: option `txqueuelen' requires an argument\n",
-	       program_name);
+      error (0, 0, "option `txqueuelen' requires an argument");
       break;
 
+    case EXPECT_INET:
     case EXPECT_NOTHING:
-      break;
+      return 1;
     }
-  if (expect != EXPECT_NOTHING)
-    usage (EXIT_FAILURE);
-
-  return 1;
+  return 0;
 }
 
 
@@ -573,8 +828,7 @@ system_configure (int sfd, struct ifreq *ifr, struct system_ifconfig *ifs)
   if (ifs->valid & IF_VALID_TXQLEN)
     {
 #ifndef SIOCSIFTXQLEN
-      printf ("%s: Don't know how to set the txqlen on this system.\n",
-	      program_name);
+      error (0, 0, "don't know how to set the txqlen on this system");
       return -1;
 #else
       int err = 0;
@@ -582,11 +836,7 @@ system_configure (int sfd, struct ifreq *ifr, struct system_ifconfig *ifs)
       ifr->ifr_qlen = ifs->txqlen;
       err = ioctl (sfd, SIOCSIFTXQLEN, ifr);
       if (err < 0)
-	{
-	  fprintf (stderr, "%s: SIOCSIFTXQLEN failed: %s\n",
-		   program_name, strerror (errno));
-	  return -1;
-	}
+	error (0, errno, "SIOCSIFTXQLEN failed");
       if (verbose)
 	printf ("Set txqlen value of `%s' to `%i'.\n",
 		ifr->ifr_name, ifr->ifr_qlen);
