@@ -1,27 +1,27 @@
-/* Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
-   Free Software Foundation, Inc.
+/*
+  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
+  2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation,
+  Inc.
 
-   This file is part of GNU Inetutils.
+  This file is part of GNU Inetutils.
 
-   GNU Inetutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
 
-   GNU Inetutils is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write
-   to the Free Software Foundation, Inc., 51 Franklin Street,
-   Fifth Floor, Boston, MA 02110-1301 USA. */
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
 /*
  * Copyright (c) 1983, 1991, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
- *  
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -30,10 +30,10 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- *  
+ *
  * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -140,6 +140,7 @@
 #include <syslog.h>
 #include <unistd.h>
 #include <argp.h>
+#include <argp-version-etc.h>
 #include <progname.h>
 #ifdef HAVE_SYS_SELECT_H
 # include <sys/select.h>
@@ -186,31 +187,27 @@ enum {
   OPT_RESOLVE
 };
 
+const char *program_authors[] = {
+  "Alain Magloire", "Alfred M. Szmidt", "Debarshi Ray",
+  "Jakob 'sparky' Kaivo", "Jeff Bailey",
+  "Jeroen Dekkers", "Marcus Brinkmann", "Sergey Poznyakoff",
+  "others", NULL };
+
 static struct argp_option argp_options[] = {
 #define GRP 0
   {"debug", 'd', NULL, 0,
-   "Turn on debugging, run in foreground mode", GRP+1},
+   "turn on debugging, run in foreground mode", GRP+1},
   {"environment", OPT_ENVIRON, NULL, 0,
-   "Pass local and remote socket information in environment variables", GRP+1},
+   "pass local and remote socket information in environment variables", GRP+1},
   {"rate", 'R', "NUMBER", 0,
-   "Maximum invocation rate (per minute)", GRP+1},
+   "maximum invocation rate (per minute)", GRP+1},
   {"resolve", OPT_RESOLVE, NULL, 0,
-   "Resolve IP addresses when setting environment variables "
+   "resolve IP addresses when setting environment variables "
    "(see --environment)", GRP+1},
 #undef GRP
   {NULL}
 };
 
-static void
-inetd_version (FILE *stream, struct argp_state *state)
-{
-  version_etc (stream, "inetd", PACKAGE_NAME, VERSION,
-	       "Alain Magloire", "Alfred M. Szmidt", "Debarshi Ray",
-	       "Jakob 'sparky' Kaivo", "Jeff Bailey",
-	       "Jeroen Dekkers", "Marcus Brinkmann", "Sergey Poznyakoff",
-	       "others", NULL);
-}
-
 static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
 {
@@ -810,8 +807,10 @@ inetd_getaddrinfo (struct servtab *sep, int proto, struct addrinfo **result)
   memset (&hints, 0, sizeof (hints));
 
   hints.ai_flags = AI_PASSIVE;
+#ifdef AI_V4MAPPED
   if (sep->se_v4mapped)
     hints.ai_flags |= AI_V4MAPPED;
+#endif
   hints.ai_family = sep->se_family;
   hints.ai_socktype = sep->se_socktype;
   hints.ai_protocol = proto;
@@ -965,16 +964,11 @@ endconfig (FILE *fconfig)
 void
 freeconfig (struct servtab *cp)
 {
-  if (cp->se_node)
-    free (cp->se_node);
-  if (cp->se_service)
-    free (cp->se_service);
-  if (cp->se_proto)
-    free (cp->se_proto);
-  if (cp->se_user)
-    free (cp->se_user);
-  if (cp->se_server)
-    free (cp->se_server);
+  free (cp->se_node);
+  free (cp->se_service);
+  free (cp->se_proto);
+  free (cp->se_user);
+  free (cp->se_server);
   argcv_free (cp->se_argc, cp->se_argv);
 }
 
@@ -1058,8 +1052,7 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
 	  if (argc == 1 && argv[0][strlen (argv[0]) - 1] == ':')
 	    {
 	      argv[0][strlen (argv[0]) - 1] = 0;
-	      if (global_serv_node)
-		free (global_serv_node);
+	      free (global_serv_node);
 	      if (strcmp (argv[0], "*"))
 		global_serv_node = newstr (argv[0]);
 	    }
@@ -1306,7 +1299,7 @@ nextconfig (const char *file)
       if (debug)
 	print_service ("FREE", sep);
       freeconfig (sep);
-      free ((char *) sep);
+      free (sep);
     }
   signal_unblock (&sigstatus);
 }
@@ -1876,6 +1869,7 @@ prepenv (int ctrl, struct sockaddr_in sa_client)
 }
 
 
+  
 int
 main (int argc, char *argv[], char *envp[])
 {
@@ -1894,7 +1888,7 @@ main (int argc, char *argv[], char *envp[])
   LastArg = envp[-1] + strlen (envp[-1]);
 
   /* Parse command line */
-  argp_program_version_hook = inetd_version;
+  iu_argp_init ("inetd", program_authors);
   argp_parse (&argp, argc, argv, 0, &index, NULL);
 
   if (resolve_option)
