@@ -1,22 +1,23 @@
-/* A version of bsd `logwtmp' that should be widely portable
+/* logwtmp.c - A version of BSDs `logwtmp' that should be widely portable
+  Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+  2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
-   Copyright (C) 1996, 2000, 2007 Free Software Foundation, Inc.
+  This file is part of GNU Inetutils.
 
-   Written by Miles Bader <miles@gnu.ai.mit.edu>
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 3, or (at
-   your option) any later version.
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+/* Written by Miles Bader.  */
 
 /* If `KEEP_OPEN' is defined, then a special version of logwtmp is compiled,
    called logwtmp_keep_open, which keeps the wtmp file descriptor open
