@@ -1,24 +1,24 @@
-/* Copyright (C) 2003, 2007, 2008 Free Software Foundation, Inc.
+/*
+  Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software
+  Foundation, Inc.
 
-This file is part of Shishi / GNU Inetutils.
+  This file is part of GNU Inetutils.
 
-Shishi is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3, or (at your option)
-any later version.
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
 
-Shishi is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with Shishi; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA. */
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
 /* Written by Simon Josefsson and Nicolas Pouvesle, based on
-   kerberos5.c from GNU InetUtils. */
+   kerberos5.c from GNU Inetutils. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
