@@ -1,3 +1,22 @@
+/*
+  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
+  2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+
+  This file is part of GNU Inetutils.
+
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
+
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
+
 /*
  * Copyright (c) 1983, 1990, 1993
  *	The Regents of the University of California.  All rights reserved.
@@ -10,7 +29,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -27,26 +46,6 @@
  * SUCH DAMAGE.
  */
 
-/* Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
-   Free Software Foundation, Inc.
-
-   This file is part of GNU Inetutils.
-
-   GNU Inetutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
-
-   GNU Inetutils is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write
-   to the Free Software Foundation, Inc., 51 Franklin Street,
-   Fifth Floor, Boston, MA 02110-1301 USA. */
-
 /*
  * rlogin - remote login
  */
@@ -232,28 +231,26 @@ extern sig_t setsig (int, sig_t);
 # define OPTIONS	"8EKde:l:hV"
 #endif
 
-ARGP_PROGRAM_DATA_SIMPLE ("rlogin", "2008");
-
 const char args_doc[] = "HOST";
 const char doc[] = "Starts a terminal session on a remote host.";
 
 static struct argp_option argp_options[] = {
 #define GRP 0
-  {"8-bit", '8', NULL, 0, "Allows an eight-bit input data path at all times",
+  {"8-bit", '8', NULL, 0, "allows an eight-bit input data path at all times",
    GRP+1},
-  {"debug", 'd', NULL, 0, "Set the SO_DEBUG option", GRP+1},
-  {"escape", 'e', "CHAR", 0, "Allows user specification of the escape "
+  {"debug", 'd', NULL, 0, "set the SO_DEBUG option", GRP+1},
+  {"escape", 'e', "CHAR", 0, "allows user specification of the escape "
    "character, which is ``~'' by default", GRP+1},
-  {"no-escape", 'E', NULL, 0, "Stops any character from being recognized as "
+  {"no-escape", 'E', NULL, 0, "stops any character from being recognized as "
    "an escape character", GRP+1},
-  {"user", 'l', "USER", 0, "Run as USER on the remote system", GRP+1},
+  {"user", 'l', "USER", 0, "run as USER on the remote system", GRP+1},
 #if defined(KERBEROS) || defined(SHISHI)
 #ifdef ENCRYPTION
-  {"encrypt", 'x', NULL, 0, "Turns on DES encryption for all data passed via "
+  {"encrypt", 'x', NULL, 0, "turns on DES encryption for all data passed via "
    "the rlogin session", GRP+1},
 #endif
-  {"kerberos", 'K', NULL, 0, "Turns off all Kerberos authentication", GRP+1},
-  {"realm", 'k', "REALM", 0, "Obtain tickets for the remote host in REALM "
+  {"kerberos", 'K', NULL, 0, "turns off all Kerberos authentication", GRP+1},
+  {"realm", 'k', "REALM", 0, "obtain tickets for the remote host in REALM "
    "realm instead of the remote's realm", GRP+1},
 #endif
 #undef GRP
@@ -354,6 +351,7 @@ main (int argc, char *argv[])
   }
 
   /* Parse command line */
+  iu_argp_init ("rlogin", default_program_authors);
   argp_parse (&argp, argc, argv, 0, &index, NULL);
 
   if (index < argc)
