@@ -1,22 +1,22 @@
-/* Copyright (C) 1998, 2001, 2002, 2006,
-   2007, 2008 Free Software Foundation, Inc.
+/*
+  Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software
+  Foundation, Inc.
 
-   This file is part of GNU Inetutils.
+  This file is part of GNU Inetutils.
 
-   GNU Inetutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
 
-   GNU Inetutils is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write
-   to the Free Software Foundation, Inc., 51 Franklin Street,
-   Fifth Floor, Boston, MA 02110-1301 USA. */
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -70,13 +70,15 @@
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
-#include <getopt.h>
 #ifdef HAVE_SYS_SELECT_H
 # include <sys/select.h>
 #endif
 #include <sys/ioctl.h>
 #include <sys/stat.h>		/* Needed for chmod() */
+#include <pty.h>
 
+#include <progname.h>
+#include <argp.h>
 #include <libinetutils.h>
 
 /*
@@ -158,24 +160,12 @@ struct auth_data
 };
 #endif
 
-static const char *short_options = "aD::d::hk::L:lnp:orxV";
-static struct option long_options[] = {
-  {"allow-root", no_argument, 0, 'o'},
-  {"verify-hostname", no_argument, 0, 'a'},
-  {"daemon", optional_argument, 0, 'd'},
-  {"no-rhosts", no_argument, 0, 'l'},
-  {"no-keepalive", no_argument, 0, 'n'},
-  {"local-domain", required_argument, 0, 'L'},
-  {"kerberos", optional_argument, 0, 'k'},
-  {"encrypt", no_argument, 0, 'x'},
-  {"debug", optional_argument, 0, 'D'},
-  {"help", no_argument, 0, 'h'},
-  {"version", no_argument, 0, 'V'},
-  {"port", required_argument, 0, 'p'},
-  {"reverse-required", no_argument, 0, 'r'},
-  {0, 0, 0, 0}
-};
+#define MODE_INETD 0
+#define MODE_DAEMON 1
+int mode = MODE_INETD;
 
+int port = 0;
+int maxchildren = DEFMAXCHILDREN;
 int allow_root = 0;
 int verify_hostname = 0;
 int keepalive = 1;
@@ -200,7 +190,6 @@ int local_dot_count;
 
 struct winsize win = { 0, 0, 0, 0 };
 
-void usage (void);
 void rlogin_daemon (int maxchildren, int port);
 int rlogind_auth (int fd, struct auth_data *ap);
 void setup_tty (int fd, struct auth_data *ap);
@@ -227,9 +216,6 @@ rlogind_sigchld (int sig)
   signal (sig, rlogind_sigchld);
 }
 
-#define MODE_INETD 0
-#define MODE_DAEMON 1
-
 #if defined(KERBEROS) && defined (ENCRYPTION)
 # define ENCRYPT_IO encrypt_io
 # define IF_ENCRYPT(stmt) if (encrypt_io) stmt
@@ -266,96 +252,134 @@ rlogind_sigchld (int sig)
 # define ENC_WRITE(c, fd, buf, size, ap) c = write (fd, buf, size)
 #endif
 
-char *program_name;
+
+const char doc[] = "Remote login server";
+const char *program_authors[] = {
+  "Alain Magloire",
+  "Sergey Poznyakoff",
+  NULL
+};
+
+static struct argp_option options[] = {
+  { "allow-root", 'o', NULL, 0,
+    "allow uid == 0 to login, disabled by default" },
+  { "verify-hostname", 'a', NULL, 0,
+    "ask hostname for verification" },
+  { "daemon", 'd', NULL, 0,
+    "daemon mode" },
+  { "no-rhosts", 'l', NULL, 0,
+    "ignore .rhosts file" },
+  { "no-keepalive", 'n', NULL, 0,
+    "do not set SO_KEEPALIVE" },
+  { "local-domain", 'L', "NAME", 0,
+    "set local domain name" },
+#if defined(KERBEROS) || defined(SHISHI)
+  { "kerberos", 'k', NULL, 0,
+    "use kerberos IV/V authentication" },
+#endif
+#if defined(ENCRYPTION)
+  { "encrypt", 'x', NULL, 0,
+    "use DES encryption" },
+#endif
+  { "debug", 'D', "LEVEL", OPTION_ARG_OPTIONAL,
+    "set debug level" },
+  { "port", 'p', "PORT", 0,
+    "listen on given port (valid only in daemon mode)" },
+  { "reverse-required", 'r', NULL, 0,
+    "require reverse resolving of a remote host IP" },
+  { NULL }
+};
+
+static error_t
+parse_opt (int key, char *arg, struct argp_state *state)
+{
+  switch (key)
+    {
+    case 'a':
+      verify_hostname = 1;
+      break;
+
+    case 'D':
+      if (arg)
+	debug_level = strtoul (arg, NULL, 10);
+      break;
+
+    case 'd':
+      mode = MODE_DAEMON;
+      if (arg)
+	maxchildren = strtoul (arg, NULL, 10);
+      if (maxchildren == 0)
+	maxchildren = DEFMAXCHILDREN;
+      break;
+
+    case 'l':
+      __check_rhosts_file = 0;	/* FIXME: extern var? */
+      break;
+
+    case 'L':
+      local_domain_name = arg;
+      break;
+
+    case 'n':
+      keepalive = 0;
+      break;
+
+#if defined(KERBEROS) || defined(SHISHI)
+    case 'k':
+      if (arg)
+	{
+	  if (*arg == '4')
+	    kerberos = AUTH_KERBEROS_4;
+	  else if (*arg == '5')
+	    kerberos = AUTH_KERBEROS_5;
+	}
+      else
+	kerberos = AUTH_KERBEROS_DEFAULT;
+      break;
+
+# ifdef ENCRYPTION
+    case 'x':
+      encrypt_io = 1;
+      break;
+# endif	/* ENCRYPTION */
+#endif /* KERBEROS */
+
+    case 'o':
+      allow_root = 1;
+      break;
+
+    case 'p':
+      port = strtoul (arg, NULL, 10);
+      break;
+      
+    case 'r':
+      reverse_required = 1;
+      break;
+      
+    default:
+      return ARGP_ERR_UNKNOWN;
+    }
+
+  return 0;
+}
+
+static struct argp argp = { options, parse_opt, NULL, doc};
+
+
 
 int
 main (int argc, char *argv[])
 {
-  int port = 0;
-  int maxchildren = DEFMAXCHILDREN;
-  int mode = MODE_INETD;
-  int c;
-
-  program_name = argv[0];
-  while ((c = getopt_long (argc, argv, short_options, long_options, NULL))
-	 != EOF)
-    {
-      switch (c)
-	{
-	case 'a':
-	  verify_hostname = 1;
-	  break;
-
-	case 'D':
-	  if (optarg)
-	    debug_level = strtoul (optarg, NULL, 10);
-	  break;
-
-	case 'd':
-	  mode = MODE_DAEMON;
-	  if (optarg)
-	    maxchildren = strtoul (optarg, NULL, 10);
-	  if (maxchildren == 0)
-	    maxchildren = DEFMAXCHILDREN;
-	  break;
-
-	case 'l':
-	  __check_rhosts_file = 0;	/* FIXME: extern var? */
-	  break;
-
-	case 'L':
-	  local_domain_name = optarg;
-	  break;
-
-	case 'n':
-	  keepalive = 0;
-	  break;
-
-#if defined(KERBEROS) || defined(SHISHI)
-	case 'k':
-	  if (optarg)
-	    {
-	      if (*optarg == '4')
-		kerberos = AUTH_KERBEROS_4;
-	      else if (*optarg == '5')
-		kerberos = AUTH_KERBEROS_5;
-	    }
-	  else
-	    kerberos = AUTH_KERBEROS_DEFAULT;
-	  break;
-
-# ifdef ENCRYPTION
-	case 'x':
-	  encrypt_io = 1;
-	  break;
-# endif	/* ENCRYPTION */
-#endif /* KERBEROS */
-
-	case 'o':
-	  allow_root = 1;
-	  break;
-
-	case 'p':
-	  port = strtoul (optarg, NULL, 10);
-	  break;
-
-	case 'r':
-	  reverse_required = 1;
-	  break;
-
-	case 'V':
-	  printf ("rlogind (%s %s)\n", PACKAGE_NAME, PACKAGE_VERSION);
-	  exit (0);
-
-	case 'h':
-	default:
-	  usage ();
-	  exit (0);
-	}
-    }
+  int index;
 
+  set_program_name (argv[0]);
+  
+  /* Parse command line */
+  iu_argp_init ("rlogind", program_authors);
+  argp_parse (&argp, argc, argv, 0, &index, NULL);
+  
   openlog ("rlogind", LOG_PID | LOG_CONS, LOG_AUTH);
-  argc -= optind;
+  argc -= index;
   if (argc > 0)
     {
       syslog (LOG_ERR, "%d extra arguments", argc);
@@ -1522,31 +1546,3 @@ fatal (int f, const char *msg, int syserr)
   write (f, buf, bp + len - buf);
   exit (1);
 }
-
-static const char usage_str[] =
-  "usage: rlogind [options]\n"
-  "\n"
-  "Options are:\n"
-  "   -a, --verify-hostname   Ask hostname for verification\n"
-  "   -d, --daemon            Daemon mode\n"
-  "   -l, --no-rhosts         Ignore .rhosts file\n"
-  "   -L, --local-domain=NAME Set local domain name\n"
-  "   -n, --no-keepalive      Do not set SO_KEEPALIVE\n"
-#if defined(KERBEROS) || defined(SHISHI)
-  "   -k, --kerberos          Use kerberos IV/V authentication\n"
-# ifdef ENCRYPTION
-  "   -x, --encrypt           Use DES encryption\n"
-# endif	/* ENCRYPTION */
-#endif /* KERBEROS */
-  "   -D, --debug[=LEVEL]     Set debug level\n"
-  "   -h, --help              Display usage instructions\n"
-  "   -V, --version           Display program version\n"
-  "   -o, --allow-root        Allow uid == 0 to login, disable by default\n"
-  "   -p, --port PORT         Listen on given port (valid only in daemon mode)\n"
-  "   -r, --reverse-required  Require reverse resolving of a remote host IP\n";
-
-void
-usage ()
-{
-  printf ("%s\n" "Send bug reports to %s\n", usage_str, PACKAGE_BUGREPORT);
-}
