@@ -1,4 +1,24 @@
-/*-
+/*
+  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
+  2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation,
+  Inc.
+
+  This file is part of GNU Inetutils.
+
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
+
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
+
+/*
  * Copyright (c) 1988, 1989, 1992, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
  *
@@ -10,7 +30,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -27,26 +47,6 @@
  * SUCH DAMAGE.
  */
 
-/* Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
-   Free Software Foundation, Inc.
-
-   This file is part of GNU Inetutils.
-
-   GNU Inetutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
-
-   GNU Inetutils is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write
-   to the Free Software Foundation, Inc., 51 Franklin Street,
-   Fifth Floor, Boston, MA 02110-1301 USA. */
-
 /*
  * remote shell server:
  *	[port]\0
@@ -103,23 +103,19 @@ char *alloca ();
 #endif
 #include <pwd.h>
 #include <signal.h>
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-# include <stdarg.h>
-#else
-# include <varargs.h>
-#endif
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
 #include <unistd.h>
-#include <getopt.h>
 #include <grp.h>
 #ifdef HAVE_SYS_SELECT_H
 # include <sys/select.h>
 #endif
 #include <error.h>
 #include <progname.h>
+#include <argp.h>
 #include <libinetutils.h>
 
 int keepalive = 1;		/* flag for SO_KEEPALIVE scoket option */
@@ -132,8 +128,6 @@ void rshd_error (const char *, ...);
 char *getstr (const char *);
 int local_domain (const char *);
 const char *topdomain (const char *);
-void usage (void);
-void help (void);
 
 #if defined(KERBEROS) || defined(SHISHI)
 # ifdef KERBEROS
@@ -154,93 +148,98 @@ int protocol;
 # endif
 # define VERSION_SIZE	9
 # define SECURE_MESSAGE  "This rsh session is using DES encryption for all transmissions.\r\n"
-# define OPTIONS		"alnkvxLVh"
 int doencrypt, use_kerberos, vacuous;
 #else
-# define OPTIONS	"alnLVh"
 #endif
 
-static const char *short_options = OPTIONS;
-static struct option long_options[] = {
-  {"verify-hostname", no_argument, 0, 'a'},
-  {"no-rhosts", no_argument, 0, 'l'},
-  {"no-keepalive", no_argument, 0, 'n'},
-  {"log-sessions", required_argument, 0, 'L'},
-  {"kerberos", no_argument, 0, 'k'},
-  {"vacuous", no_argument, 0, 'v'},
-  {"help", no_argument, 0, 'h'},
-  {"version", no_argument, 0, 'V'},
-  {0, 0, 0, 0}
+static struct argp_option options[] = {
+  { "verify-hostname", 'a', NULL, 0,
+    "ask hostname for verification" },
+  { "no-rhosts", 'l', NULL, 0,
+    "ignore .rhosts file" },
+  { "no-keepalive", 'n', NULL, 0,
+    "do not set SO_KEEPALIVE" },
+  { "log-sessions", 'L', NULL, 0,
+    "log successfull logins" },
+#ifdef	KERBEROS
+  /* FIXME: The option semantics does not match that of others r* utilities */
+  { "kerberos", 'k', NULL, 0,
+    "use kerberos IV authentication" },
+  /* FIXME: Option name is misleading */
+  { "vacuous", 'v', NULL, 0,
+    "require Kerberos authentication" },
+#endif
+  { NULL }
 };
 
+extern int __check_rhosts_file;	/* hook in rcmd(3) */
+
+static error_t
+parse_opt (int key, char *arg, struct argp_state *state)
+{
+  switch (key)
+    {
+    case 'a':
+      check_all = 1;
+      break;
+
+    case 'l':
+      __check_rhosts_file = 0;	/* don't check .rhosts file */
+      break;
+
+    case 'n':
+      keepalive = 0;	/* don't enable SO_KEEPALIVE */
+      break;
+
+#if defined(KERBEROS) || defined(SHISHI)
+    case 'k':
+      use_kerberos = 1;
+      break;
+
+    case 'v':
+      vacuous = 1;
+      break;
+
+# ifdef ENCRYPTION
+    case 'x':
+      doencrypt = 1;
+      break;
+# endif
+#endif
+
+    case 'L':
+      log_success = 1;
+      break;
+      
+    default:
+      return ARGP_ERR_UNKNOWN;
+    }
+
+  return 0;
+}
+
+
+const char doc[] = "Remote shell server";
+static struct argp argp = { options, parse_opt, NULL, doc};
+
+
 /* Remote shell server. We're invoked by the rcmd(3) function. */
 int
 main (int argc, char *argv[])
 {
-  extern int __check_rhosts_file;	/* hook in rcmd(3) */
+  int index;
   struct linger linger;
   int ch, on = 1, fromlen;
   struct sockaddr_in from;
   int sockfd;
 
   set_program_name (argv[0]);
+  iu_argp_init ("rshd", default_program_authors);
+  argp_parse (&argp, argc, argv, 0, &index, NULL);
   
-  opterr = 0;
-  while ((ch = getopt_long (argc, argv, short_options, long_options, NULL))
-	 != EOF)
-    {
-      switch (ch)
-	{
-	case 'a':
-	  check_all = 1;
-	  break;
-
-	case 'l':
-	  __check_rhosts_file = 0;	/* don't check .rhosts file */
-	  break;
-
-	case 'n':
-	  keepalive = 0;	/* don't enable SO_KEEPALIVE */
-	  break;
-
-#if defined(KERBEROS) || defined(SHISHI)
-	case 'k':
-	  use_kerberos = 1;
-	  break;
-
-	case 'v':
-	  vacuous = 1;
-	  break;
-
-# ifdef ENCRYPTION
-	case 'x':
-	  doencrypt = 1;
-	  break;
-# endif
-#endif
-
-	case 'L':
-	  log_success = 1;
-	  break;
-
-	case 'V':
-	  printf ("rshd (%s %s)\n", PACKAGE_NAME, PACKAGE_VERSION);
-	  exit (0);
-
-	case 'h':
-	  help ();
-	  exit (0);
-
-	case '?':
-	default:
-	  usage ();
-	  break;
-	}
-    }
-
   openlog ("rshd", LOG_PID | LOG_ODELAY, LOG_DAEMON);
 
-  argc -= optind;
+  argc -= index;
   if (argc > 0)
     {
       syslog (LOG_ERR, "%d extra arguments", argc);
@@ -507,7 +506,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 #endif
 	if (check_all || local_domain (hp->h_name))
 	  {
-	    char *remotehost = (char *) alloca (strlen (hostname) + 1);
+	    char *remotehost = alloca (strlen (hostname) + 1);
 	    if (!remotehost)
 	      errorstr = "Out of memory\n";
 	    else
@@ -1262,36 +1261,3 @@ topdomain (const char *h)
   return maybe;
 }
 
-void
-usage ()
-{
-  syslog (LOG_ERR, "usage: rshd [-%s]", OPTIONS);
-  exit (2);
-}
-
-void
-help (void)
-{
-  puts ("usage: rshd [options]");
-  puts ("\
-   -a, --verify-hostname   Ask hostname for verification");
-  puts ("\
-   -l, --no-rhosts         Ignore .rhosts file");
-  puts ("\
-   -L, --log-session       Set local domain name");
-  puts ("\
-   -n, --no-keepalive      Do not set SO_KEEPALIVE");
-#if defined(KERBEROS) || defined(SHISHI)
-  puts ("\
-   -k, --kerberos          Use kerberos IV authentication");
-# ifdef ENCRYPTION
-  puts ("\
-   -x, --encrypt           Use DES encryption");
-# endif	/* ENCRYPTION */
-#endif /* KERBEROS */
-  puts ("\
-   -h, --help              Display usage instructions");
-  puts ("\
-   -V, --version           Display program version");
-  printf ("\nSend bug reports to %s\n", PACKAGE_BUGREPORT);
-}
