@@ -1,5 +1,23 @@
 /* syslogd - log system messages
- *
+  Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+  2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+
+  This file is part of GNU Inetutils.
+
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
+
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
+
+/*
  * Copyright (c) 1983, 1988, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
  *
@@ -11,7 +29,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -28,27 +46,6 @@
  * SUCH DAMAGE.
  */
 
-/* Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
-   Free Software Foundation, Inc.
-
-   This file is part of GNU Inetutils.
-
-   GNU Inetutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
-
-   GNU Inetutils is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write
-   to the Free Software Foundation, Inc., 51 Franklin Street,
-   Fifth Floor, Boston, MA 02110-1301 USA. */
-
-
 /*
  *  syslogd -- log system messages
  *
@@ -141,7 +138,7 @@
 /* A mask of all facilities mentioned explicitly in the configuration file
  *
  * This is used to support a virtual facility "**" that covers all the rest,
- * so that messages to unexpected facilities won't be lost when "*" is 
+ * so that messages to unexpected facilities won't be lost when "*" is
  * not logged to a file.
  */
 int facilities_seen;
@@ -302,8 +299,6 @@ time_t now;			/* Time use for mark and forward supending.  */
 int force_sync;			/* GNU/Linux behaviour to sync on every line.
 				   This off by default. Set to 1 to enable.  */
 
-ARGP_PROGRAM_DATA_SIMPLE ("syslogd", "2008");
-
 const char args_doc[] = "";
 const char doc[] = "Log system messages.";
 
@@ -318,34 +313,34 @@ enum {
 static struct argp_option argp_options[] = {
 #define GRP 0
   /* Not sure about the long name. Maybe move into conffile even. */
-  {NULL, 'a', "SOCKET", 0, "Add unix socket to listen to (up to 19)", GRP+1},
-  {NULL, 'l', "HOSTLIST", 0, "Log hosts in HOSTLIST by their hostname", GRP+1},
-  {NULL, 's', "DOMAINLIST", 0, "List of domains which should be stripped "
+  {NULL, 'a', "SOCKET", 0, "add unix socket to listen to (up to 19)", GRP+1},
+  {NULL, 'l', "HOSTLIST", 0, "log hosts in HOSTLIST by their hostname", GRP+1},
+  {NULL, 's', "DOMAINLIST", 0, "list of domains which should be stripped "
    "from the FQDN of hosts before logging their name", GRP+1},
-  {"debug", 'd', NULL, 0, "Print debug information (implies --no-detach)",
+  {"debug", 'd', NULL, 0, "print debug information (implies --no-detach)",
    GRP+1},
-  {"hop", 'h', NULL, 0, "Forward messages from remote hosts", GRP+1},
-  {"inet", 'r', NULL, 0, "Receive remote messages via internet domain socket",
+  {"hop", 'h', NULL, 0, "forward messages from remote hosts", GRP+1},
+  {"inet", 'r', NULL, 0, "receive remote messages via internet domain socket",
    GRP+1},
-  {"mark", 'm', "INTVL", 0, "Specify timestamp interval in logs (0 for no "
+  {"mark", 'm', "INTVL", 0, "specify timestamp interval in logs (0 for no "
    "timestamps)", GRP+1},
-  {"no-detach", 'n', NULL, 0, "Don't enter daemon mode", GRP+1},
-  {"no-forward", OPT_NO_FORWARD, NULL, 0, "Do not forward any messages "
+  {"no-detach", 'n', NULL, 0, "do not enter daemon mode", GRP+1},
+  {"no-forward", OPT_NO_FORWARD, NULL, 0, "do not forward any messages "
    "(overrides --hop)", GRP+1},
 #ifdef PATH_KLOG
-  {"no-klog", OPT_NO_KLOG, NULL, 0, "Do not listen to kernel log device "
+  {"no-klog", OPT_NO_KLOG, NULL, 0, "do not listen to kernel log device "
    PATH_KLOG, GRP+1},
 #endif
-  {"no-unixaf", OPT_NO_UNIXAF, NULL, 0, "Do not listen on unix domain "
+  {"no-unixaf", OPT_NO_UNIXAF, NULL, 0, "do not listen on unix domain "
    "sockets (overrides -a and -p)", GRP+1},
-  {"pidfile", OPT_PIDFILE, "FILE", 0, "Override pidfile (default: "
+  {"pidfile", OPT_PIDFILE, "FILE", 0, "override pidfile (default: "
    PATH_LOGPID ")", GRP+1},
-  {"rcfile", 'f', "FILE", 0, "Override configuration file (default: "
+  {"rcfile", 'f', "FILE", 0, "override configuration file (default: "
    PATH_LOGCONF ")",
    GRP+1},
-  {"socket", 'p', "FILE", 0, "Override default unix domain socket " PATH_LOG,
+  {"socket", 'p', "FILE", 0, "override default unix domain socket " PATH_LOG,
    GRP+1},
-  {"sync", 'S', NULL, 0, "Force a file sync on every line", GRP+1},
+  {"sync", 'S', NULL, 0, "force a file sync on every line", GRP+1},
 #undef GRP
   {NULL}
 };
@@ -446,11 +441,12 @@ main (int argc, char *argv[])
   unsigned long nfds = 0;
 
   set_program_name (argv[0]);
-  
+
   /* Initiliaze PATH_LOG as the first element of the unix sockets array.  */
   add_funix (PATH_LOG);
 
   /* Parse command line */
+  iu_argp_init ("syslogd", default_program_authors);
   argp_parse (&argp, argc, argv, 0, NULL, NULL);
 
   /* Daemonise, if not, set the buffering for line buffer.  */
@@ -495,8 +491,7 @@ main (int argc, char *argv[])
       if (host_ent)
 	{
 	  /* Override what we had */
-	  if (LocalHostName)
-	    free (LocalHostName);
+	  free (LocalHostName);
 	  LocalHostName = strdup (host_ent->h_name);
 	  p = strchr (LocalHostName, '.');
 	  if (p != NULL)
@@ -663,8 +658,8 @@ main (int argc, char *argv[])
 		      }
 
 		    /* This loop makes sure the daemon won't lock up
-		     * on null bytes in the klog stream.  They still hurt 
-		     * efficiency, acting like a message separator that 
+		     * on null bytes in the klog stream.  They still hurt
+		     * efficiency, acting like a message separator that
 		     * forces a shift-and-reiterate when the buffer was
 		     * never full.
 		     */
@@ -679,7 +674,7 @@ main (int argc, char *argv[])
 
 		    if (bol != kline)
 		      {
-			/* shift the partial line to start of buffer, so 
+			/* shift the partial line to start of buffer, so
 			 * we can re-iterate.
 			 */
 			memmove (kline, bol, kline_len);
@@ -695,11 +690,11 @@ main (int argc, char *argv[])
 			 */
 			printsys (kline);
 
-			/* Clone priority signal if present 
+			/* Clone priority signal if present
 			 * We merely shift the kline_len pointer after
 			 * it so the next chunk is written after it.
 			 *
-			 * strchr(kline,'>') is not used as it would allow 
+			 * strchr(kline,'>') is not used as it would allow
 			 * a pathological line ending in '>' to cause an
 			 * endless loop.
 			 */
@@ -1107,8 +1102,7 @@ logmsg (int pri, const char *msg, const char *from, int flags)
 	    fprintlog (f, from, 0, (char *) NULL);
 	  f->f_repeatcount = 0;
 	  strncpy (f->f_lasttime, timestamp, sizeof (f->f_lasttime) - 1);
-	  if (f->f_prevhost)
-	    free (f->f_prevhost);
+	  free (f->f_prevhost);
 	  f->f_prevhost = strdup (from);
 	  if (msglen < MAXSVLINE)
 	    {
@@ -1860,7 +1854,7 @@ cfline (const char *line, struct filed *f)
 	  if (*buf == '*')
 	    for (i = 0; i <= LOG_NFACILITIES; i++)
 	      {
-		/* make "**" act as a wildcard only for facilities not 
+		/* make "**" act as a wildcard only for facilities not
 		 * specified elsewhere
 		 */
 		if (buf[1] == '*' && ((1 << i) & facilities_seen))
