@@ -1,53 +1,33 @@
-/* Copyright (C) 1998, 2001, 2002, 2004, 2006, 2007 Free Software Foundation, Inc.
+/*
+  Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software
+  Foundation, Inc.
 
-   This file is part of GNU Inetutils.
+  This file is part of GNU Inetutils.
 
-   GNU Inetutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
 
-   GNU Inetutils is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write
-   to the Free Software Foundation, Inc., 51 Franklin Street,
-   Fifth Floor, Boston, MA 02110-1301 USA. */
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
 #include "telnetd.h"
 
-#include <getopt.h>
-
 #ifdef HAVE_SYS_UTSNAME_H
 # include <sys/utsname.h>
 #endif
+#include <argp.h>
+#include <progname.h>
+#include <error.h>
 #include <libinetutils.h>
 
-static char short_options[] = "a:D::d:E:HhLl::nS:Uu:VX:";
-
-static struct option long_options[] = {
-  /* Help options */
-  {"version", no_argument, NULL, 'V'},
-  {"license", no_argument, NULL, 'L'},
-  {"help", no_argument, NULL, 'H'},
-  /* Common options */
-  {"authmode", required_argument, NULL, 'a'},
-  {"debug", optional_argument, NULL, 'D'},
-  {"exec-login", required_argument, NULL, 'E'},
-  {"no-hostinfo", no_argument, NULL, 'h'},
-  {"linemode", optional_argument, NULL, 'l'},
-  {"no-keepalive", no_argument, NULL, 'n'},
-  {"reverse-lookup", no_argument, NULL, 'U'},
-  {"disable-auth-type", required_argument, NULL, 'X'},
-  {NULL, 0, NULL, 0}
-};
-
-static void telnetd_version (void);
-static void telnetd_license (void);
-static void telnetd_help (void);
 static void parse_authmode (char *str);
 static void parse_linemode (char *str);
 static void parse_debug_level (char *str);
@@ -108,74 +88,98 @@ char *terminaltype;
 
 int SYNCHing;			/* we are in TELNET SYNCH mode */
 struct telnetd_clocks clocks;
-char *program_name;
+
+
+static struct argp_option argp_options[] = {
+  { "authmode", 'a', "MODE", 0,
+    "specify what mode to use for authentication" },
+  { "debug", 'D', "LEVEL", OPTION_ARG_OPTIONAL,
+    "set debugging level" },
+  { "exec-login", 'E', "STRING", 0,
+    "set program to be executed instead of /bin/login" },
+  { "no-hostinfo", 'h', NULL, 0,
+    "do not print host information before login has been completed" },
+  { "linemode", 'l', "MODE", OPTION_ARG_OPTIONAL,
+    "set line mode" },
+  { "no-keepalive", 'n', NULL, 0,
+    "disable TCP keep-alives" },
+  { "reverse-lookup", 'U', NULL, 0,
+    "refuse  connections from addresses that "
+    "cannot be mapped back into a symbolic name" },
+#ifdef  AUTHENTICATION
+  { "disable-auth-type", 'X', "TYPE", 0,
+    "disable the use of given authentication option" },
+#endif
+  { NULL }
+};
+
+static error_t
+parse_opt (int key, char *arg, struct argp_state *state)
+{
+  switch (key)
+    {
+    case 'a':
+      parse_authmode (arg);
+      break;
+
+    case 'D':
+      parse_debug_level (arg);
+      break;
+
+    case 'E':
+      login_invocation = arg;
+      break;
+	  
+    case 'h':
+      hostinfo = 0;
+      break;
+
+    case 'l':
+      parse_linemode (arg);
+      break;
+
+    case 'n':
+      keepalive = 0;
+      break;
+	  
+    case 'U':
+      reverse_lookup = 1;
+      break;
+
+#ifdef	AUTHENTICATION
+    case 'X':
+      auth_disable_name (arg);
+      break;
+#endif
+      
+    default:
+      return ARGP_ERR_UNKNOWN;
+    }
+
+  return 0;
+}
+
+static struct argp argp =
+  {
+    argp_options,
+    parse_opt,
+    NULL,
+    "DARPA telnet protocol server"
+  };
+
+
 
 int
 main (int argc, char **argv)
 {
-  int c;
-  program_name = argv[0];
-  while ((c = getopt_long (argc, argv, short_options, long_options, NULL))
-	 != EOF)
-    {
-      switch (c)
-	{
-	case 'V':
-	  telnetd_version ();
-	  exit (0);
+  int index;
 
-	case 'L':
-	  telnetd_license ();
-	  exit (0);
-
-	case 'H':
-	  telnetd_help ();
-	  exit (0);
-
-	case 'a':
-	  parse_authmode (optarg);
-	  break;
-
-	case 'D':
-	  parse_debug_level (optarg);
-	  break;
-
-	case 'E':
-	  login_invocation = optarg;
-	  break;
-
-	case 'h':
-	  hostinfo = 0;
-	  break;
-
-	case 'l':
-	  parse_linemode (optarg);
-	  break;
-
-	case 'n':
-	  keepalive = 0;
-	  break;
-
-	case 'U':
-	  reverse_lookup = 1;
-	  break;
-
-#ifdef	AUTHENTICATION
-	case 'X':
-	  auth_disable_name (optarg);
-	  break;
-#endif
-	default:
-	  fprintf (stderr, "telnetd: unknown command line option: %c\n", c);
-	  exit (1);
-	}
-    }
-
-  if (argc != optind)
-    {
-      fprintf (stderr, "telnetd: junk arguments in the command line\n");
-      exit (1);
-    }
+  set_program_name (argv[0]);
+  iu_argp_init ("telnetd", default_program_authors);
+  argp_parse (&argp, argc, argv, 0, &index, NULL);
+  
+  if (argc != index)
+    error (1, 0, "junk arguments in the command line");
 
   openlog ("telnetd", LOG_PID | LOG_ODELAY, LOG_DAEMON);
   telnetd_setup (0);
@@ -686,69 +690,3 @@ print_hostinfo ()
   pty_input_putback (str, strlen (str));
   free (str);
 }
-
-void
-telnetd_version ()
-{
-  printf ("telnetd - %s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
-  printf ("Copyright (C) 1998,2001,2002 Free Software Foundation, Inc.\n");
-  printf ("%s comes with ABSOLUTELY NO WARRANTY.\n", PACKAGE_NAME);
-  printf ("You may redistribute copies of %s\n", PACKAGE_NAME);
-  printf ("under the terms of the GNU General Public License.\n");
-  printf ("For more information about these matters, ");
-  printf ("see the files named COPYING.\n");
-}
-
-void
-telnetd_license ()
-{
-  static char license_text[] =
-    "   This program is free software; you can redistribute it and/or modify\n"
-    "   it under the terms of the GNU General Public License as published by\n"
-    "   the Free Software Foundation; either version 3, or (at your option)\n"
-    "   any later version.\n"
-    "\n"
-    "   This program is distributed in the hope that it will be useful,\n"
-    "   but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-    "   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-    "   GNU General Public License for more details.\n"
-    "\n"
-    "   You should have received a copy of the GNU General Public License\n"
-    "   along with this program; if not, write to the Free Software\n"
-    "   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n";
-  printf ("%s", license_text);
-}
-
-void
-telnetd_help ()
-{
-  printf ("\
-Usage: telnetd [OPTION]\n\
-\n\
-Options are:\n\
-    -a, --authmode AUTHMODE  specify what mode to use for authentication\n\
-    -D, --debug[=LEVEL]      set debugging level\n\
-    -E, --exec-login STRING  set program to be executed instead of /bin/login\n\
-    -h, --no-hostinfo        do not print host information before login has\n\
-                             been completed\n\
-    -l, --linemode[=MODE]    set line mode\n\
-    -n, --no-keepalive       disable TCP keep-alives\n\
-    -U, --reverse-lookup     refuse  connections from addresses that\n\
-                             cannot be mapped back into a symbolic name\n\
-    -X, --disable-auth-type AUTHTYPE\n\
-                             disable the use of given authentication option\n\
-Informational options:\n\
-    -V, --version         display this help and exit\n\
-    -L, --license	  display license and exit\n\
-    -H. --help		  output version information and exit\n");
-}
-
-int
-stop ()
-{
-  int volatile _s = 1;
-
-  while (_s)
-    _s = _s;
-  return 0;
-}
