@@ -1,21 +1,22 @@
-/* Copyright (C) 1998,2001,2005,2006,2007 Free Software Foundation, Inc.
+/*
+  Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software
+  Foundation, Inc.
 
-   This file is part of GNU Inetutils.
+  This file is part of GNU Inetutils.
 
-   GNU Inetutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
 
-   GNU Inetutils is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write
-   to the Free Software Foundation, Inc., 51 Franklin Street,
-   Fifth Floor, Boston, MA 02110-1301 USA. */
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
 #define TELOPTS
 #define TELCMDS
@@ -759,14 +760,12 @@ getterminaltype (char *user_name)
        */
       if (his_state_is_will (TELOPT_TTYPE) && !terminaltypeok (terminaltype))
 	{
-	  if (first)
-	    free (first);
+	  free (first);
 	  first = xstrdup (terminaltype);
 	  for (;;)
 	    {
 	      /* Save the unknown name, and request the next name. */
-	      if (last)
-		free (last);
+	      free (last);
 	      last = xstrdup (terminaltype);
 	      _gettermname ();
 	      if (terminaltypeok (terminaltype))
@@ -795,10 +794,8 @@ getterminaltype (char *user_name)
 		}
 	    }
 	}
-      if (first)
-	free (first);
-      if (last)
-	free (last);
+      free (first);
+      free (last);
     }
   return retval;
 }
