@@ -1,3 +1,22 @@
+/*
+  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
+  2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+
+  This file is part of GNU Inetutils.
+
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
+
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
+
 /*
  * Copyright (c) 1983, 1993
  *	The Regents of the University of California.  All rights reserved.
@@ -10,7 +29,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -53,7 +72,6 @@
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <getopt.h>
 #include <netdb.h>
 #include <setjmp.h>
 #include <signal.h>
@@ -65,6 +83,10 @@
 
 #include "tftpsubs.h"
 
+#include <argp.h>
+#include <progname.h>
+#include <libinetutils.h>
+
 void usage (void);
 
 #define TIMEOUT		5
@@ -105,53 +127,70 @@ static const char *errtomsg (int);
 static void nak (int);
 static const char *verifyhost (struct sockaddr_in *);
 
-static const char *short_options = "hVln";
-static struct option long_options[] = {
-  {"help", no_argument, 0, 'h'},
-  {"version", no_argument, 0, 'V'},
-  {0}
-};
+
 
+static struct argp_option options[] = {
+  { "logging", 'l', NULL, 0,
+    "enable logging" },
+  { "nonexistent", 'n', NULL, 0,
+    "supress negative acknowledgement of requests for "
+    "nonexistent relative filenames" },
+  { NULL }
+};
+  
+static error_t
+parse_opt (int key, char *arg, struct argp_state *state)
+{
+  switch (key)
+    {
+    case 'l':
+      logging = 1;
+      break;
+
+    case 'n':
+      suppress_naks = 1;
+      break;
+      
+    default:
+      return ARGP_ERR_UNKNOWN;
+    }
+
+  return 0;
+}
+
+static struct argp argp =
+  {
+    options,
+    parse_opt,
+    "directory...",
+    "Trivial File Transfer Protocol server"
+  };
+
+
 int
 main (int argc, char *argv[])
 {
+  int index;
   register struct tftphdr *tp;
-  register int n;
-  int ch, on;
+  int on, n;
   struct sockaddr_in sin;
 
+  set_program_name (argv[0]);
+  iu_argp_init ("tftpd", default_program_authors);
+  argp_parse (&argp, argc, argv, 0, &index, NULL);
+  
   openlog ("tftpd", LOG_PID, LOG_FTP);
-  while ((ch = getopt_long (argc, argv, short_options, long_options, NULL))
-	 != EOF)
-    {
-      switch (ch)
-	{
-	case 'l':
-	  logging = 1;
-	  break;
-	case 'n':
-	  suppress_naks = 1;
-	  break;
-	case 'V':
-	  printf ("tftpd (%s %s)\n", PACKAGE_NAME, PACKAGE_VERSION);
-	  exit (0);
-	case 'h':
-	  usage ();
-	  exit (0);
-	default:
-	  syslog (LOG_WARNING, "ignoring unknown option -%c", ch);
-	}
-    }
-  if (optind < argc)
+
+  if (index < argc)
     {
       struct dirlist *dirp;
 
       /* Get list of directory prefixes. Skip relative pathnames. */
-      for (dirp = dirs; optind < argc && dirp < &dirs[MAXDIRS]; optind++)
+      for (dirp = dirs; index < argc && dirp < &dirs[MAXDIRS]; index++)
 	{
-	  if (argv[optind][0] == '/')
+	  if (argv[index][0] == '/')
 	    {
-	      dirp->name = argv[optind];
+	      dirp->name = argv[index];
 	      dirp->len = strlen (dirp->name);
 	      dirp++;
 	    }
@@ -427,8 +466,7 @@ validate_access (char **filep, int mode)
       err = ENOTFOUND;
       for (dirp = dirs; dirp->name != NULL; dirp++)
 	{
-	  if (pathname)
-	    free (pathname);
+	  free (pathname);
 	  pathname = malloc (strlen (dirp->name) + 1 + strlen (filename) + 1);
 	  if (!pathname)
 	    return ENOMEM;
