@@ -1,10 +1,27 @@
+/*
+  Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software
+  Foundation, Inc.
+
+  This file is part of GNU Inetutils.
+
+  GNU Inetutils is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or (at
+  your option) any later version.
+
+  GNU Inetutils is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see `http://www.gnu.org/licenses/'. */
+
 /*
  * Copyright (c) 1985, 1993
  *	The Regents of the University of California.  All rights reserved.
  *
- * This code is derived from software contributed to Berkeley by
- * Rick Adams.
- *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -13,7 +30,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -30,25 +47,8 @@
  * SUCH DAMAGE.
  */
 
-/* Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
-   Free Software Foundation, Inc.
-
-   This file is part of GNU Inetutils.
-
-   GNU Inetutils is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
-
-   GNU Inetutils is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with GNU Inetutils; see the file COPYING.  If not, write
-   to the Free Software Foundation, Inc., 51 Franklin Street,
-   Fifth Floor, Boston, MA 02110-1301 USA. */
+/* This code is derived from software contributed to Berkeley by Rick
+   Adams. */
 
 /*
  * 4.2BSD TCP/IP server for uucico
@@ -80,7 +80,6 @@
 #  include <time.h>
 # endif
 #endif
-#include <getopt.h>
 #include <pwd.h>
 #include <unistd.h>
 #include <errno.h>
@@ -97,8 +96,11 @@
 # include <utmp.h>
 #endif
 
+#include <argp.h>
+#include <progname.h>
+#include <libinetutils.h>
+
 void dologin ();
-void usage (void);
 
 struct sockaddr_in hisctladdr;
 int hisaddrlen = sizeof hisctladdr;
@@ -112,61 +114,26 @@ char *nenv[] = {
 };
 extern char **environ;
 
-static const char *short_options = "hV";
-static struct option long_options[] = {
-  {"help", no_argument, 0, 'h'},
-  {"version", no_argument, 0, 'V'},
-  {0}
-};
+static struct argp argp =
+  {
+    NULL,
+    NULL,
+    NULL,
+    "TCP/IP server for uucico"
+  };
 
 int
 main (int argc, char **argv)
 {
-#ifndef BSDINETD
   register int s;
-# ifdef BSD4_2
-  register int tcp_socket;
-# endif
   struct servent *sp;
-#endif /* !BSDINETD */
-  extern int errno;
   void dologout ();
-  int c;
 
-  while ((c = getopt_long (argc, argv, short_options, long_options, NULL))
-	 != EOF)
-    {
-      switch (c)
-	{
-	case 'V':
-	  printf ("uucpd (%s %s)\n", PACKAGE_NAME, PACKAGE_VERSION);
-	  exit (0);
-
-	case 'h':
-	default:
-	  usage ();
-	  exit (0);
-	}
-    }
+  set_program_name (argv[0]);
+  iu_argp_init ("uucpd", default_program_authors);
+  argp_parse (&argp, argc, argv, 0, NULL, NULL);
 
   environ = nenv;
-#ifdef BSDINETD
-  close (1);
-  close (2);
-  dup (0);
-  dup (0);
-  hisaddrlen = sizeof (hisctladdr);
-  if (getpeername (0, &hisctladdr, &hisaddrlen) < 0)
-    {
-      fprintf (stderr, "%s: ", argv[0]);
-      perror ("getpeername");
-      _exit (1);
-    }
-  if (fork () == 0)
-    doit (&hisctladdr);
-  dologout ();
-  exit (1);
-#else /* !BSDINETD */
   sp = getservbyname ("uucp", "tcp");
   if (sp == NULL)
     {
@@ -184,49 +151,6 @@ main (int argc, char **argv)
   bzero ((char *) &myctladdr, sizeof (myctladdr));
   myctladdr.sin_family = AF_INET;
   myctladdr.sin_port = sp->s_port;
-# ifdef BSD4_2
-  tcp_socket = socket (AF_INET, SOCK_STREAM, 0);
-  if (tcp_socket < 0)
-    {
-      perror ("uucpd: socket");
-      exit (1);
-    }
-  if (bind (tcp_socket, (char *) &myctladdr, sizeof (myctladdr)) < 0)
-    {
-      perror ("uucpd: bind");
-      exit (1);
-    }
-  listen (tcp_socket, 3);	/* at most 3 simultaneuos uucp connections */
-  signal (SIGCHLD, dologout);
-
-  for (;;)
-    {
-      s = accept (tcp_socket, &hisctladdr, &hisaddrlen);
-      if (s < 0)
-	{
-	  if (errno == EINTR)
-	    continue;
-	  perror ("uucpd: accept");
-	  exit (1);
-	}
-      if (fork () == 0)
-	{
-	  close (0);
-	  close (1);
-	  close (2);
-	  dup (s);
-	  dup (s);
-	  dup (s);
-	  close (tcp_socket);
-	  close (s);
-	  doit (&hisctladdr);
-	  exit (1);
-	}
-      close (s);
-    }
-# endif	/* BSD4_2 */
-
-#endif /* !BSDINETD */
 }
 
 static int
@@ -297,21 +221,11 @@ doit (struct sockaddr_in *sinp)
   sprintf (Username, "USER=%s", user);
   dologin (pw, sinp);
   setgid (pw->pw_gid);
-#ifdef BSD4_2
-  initgroups (pw->pw_name, pw->pw_gid);
-#endif /* BSD4_2 */
   chdir (pw->pw_dir);
   setuid (pw->pw_uid);
-#ifdef BSD4_2
-  execl (UUCICO, "uucico", (char *) 0);
-#endif /* BSD4_2 */
   perror ("uucico server: execl");
 }
 
-#ifdef BSD4_2
-# include <fcntl.h>
-#endif /* BSD4_2 */
-
 void
 dologout ()
 {
@@ -359,7 +273,8 @@ dologin (struct passwd *pw, struct sockaddr_in *sin)
 
 #if defined (PATH_LASTLOG) && defined (HAVE_STRUCT_LASTLOG)
 # define SCPYN(a, b)	strncpy(a, b, sizeof (a))
-  if ((f = open (PATH_LASTLOG, O_RDWR)) >= 0)
+  f = open (PATH_LASTLOG, O_RDWR);
+  if (f >= 0)
     {
       struct lastlog ll;
 
@@ -373,16 +288,3 @@ dologin (struct passwd *pw, struct sockaddr_in *sin)
     }
 #endif
 }
-
-static const char usage_str[] =
-  "Usage: uucpd [OPTIONS...]\n"
-  "\n"
-  "Options are:\n"
-  "       --help              Display usage instructions\n"
-  "       --version           Display program version\n";
-
-void
-usage (void)
-{
-  printf ("%s\n" "Send bug reports to <%s>\n", usage_str, PACKAGE_BUGREPORT);
-}
